
!----------------------------------------------------------------------
!              SPHERE part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: initialize_mod.f90,v 1.1 2001/08/11 15:41:29 ingves Exp ingves $
!-----------------------------------------------------------------------


Module Initialize_mod

  Use Global_definitions

Contains

  !------------------------------------------------------------------
  Subroutine get_parameters(param)

    Implicit None    
    Type(parameters)   :: param

    ! Properties of the incident light
    param%polarization        = 'p'	               ! Polarization of the incident beam
    param%energy_min          = 1.5_wp	               ! Photon energy range (eV)
    param%energy_max          = 5._wp
    param%theta0              = pi/4._wp	       ! Incident angles (rad)
    param%theta0_calc	      = pi/4._wp
    param%phi0_calc           = 0._wp      


    ! Geometry of the system
    param%film_thickness1     = 3._wp
    param%film_thickness2     = 1._wp
    param%geometry            = 'island'                ! Film geometry (film/island)
    
     ! Material parameters
    param%island              = 'ag'			! Island material
    param%substrate           = 'al2o3p'                ! Substrate material
    param%coating	      = 'ti'		        ! Coating material

    ! Parameters for the dielectric constant of metal
    param%mean_free_path      = 'tau'                    ! Mean free path of electrons
    param%surface_effects     = .true.                   ! Surface effects 
    param%temperature	      = 300                      ! Temperature(K)
    param%thermoreflectance   = .false.	
    
     ! Properties of the islands
     param%R	              = 6.8_wp		         ! Island radius (nm)
     param%tr		      = 0.11_wp		         ! Truncation parameter

     ! "Island-island" parameters
     param%network	      = 'square'                 ! The type of network for interactions
     param%coverage	      = 0.3922219_wp		 ! Coverage 
     param%lattice_const      = 19.245_wp		 ! The lattice constant
     param%interaction        = 'dipole'                 ! Island-island interaction
	 
     ! Output parameters
     param%path		      = '/home/ingves/Fortran/Gransim/Dielectric' ! Path for the dielectrics

     param%outfilename	      = 'results'	         ! Output filename (basename)
     param%fresnel	      = 'constitutive'           ! Type of formulae for DR
     param%out		      = 'A'			 ! Nature of output
     param%compa	      = .false.		         ! Comparison with experiment 
     param%expfilename	      = 'experi'                 ! Experimental filename
     param%strenght	      = .false.		         ! Output for the multipoles coefficients
     param%polarizability     = .false.		         ! Output for the polarizabilities
     param%normalization      = .false.		         ! Normalization by the volume for the polarizabilty
     
     ! Boundary parameters
     param%bound	      = 'none'		         ! Nature for the boundaries conditions
     param%BC_energy          = 3.5271_wp		 ! energy for the BC
     param%scaling	      = 'incident_field'         ! Nature for the normalization value
     
     
     ! Numerics and method parameters 
     param%Nenergy	      = 50			 ! # energies  
     param%Mpole_order	      = 5		    	 ! # multipole
     param%Levels	      = 500			 ! # levels
     param%MPpos	      = 0._wp			 ! Position of the multipoles
     param%int_method	      = 'quadpack'	         ! Integration method
     param%Nint		      = 250			 ! # points in integration in Gauleg
     param%limit	      = 30			 ! Number of interval subdivision in Quadpack
     param%epsabs	      = 1e-12			 ! Absolute precison in the integrals
     param%epsrel	      = 1e-9			 ! Relative precision in integration
     param%epslin	      = 1e-4			 ! Relative error in Ax=b
     param%MP_precision	      = 20			 ! Number of digits in the MP package
      
     ! Physical constants
     param%hbar		      = 6.582122E-16             ! Plancks constant /2pi (eV s)
     param%c                  = 2.99792458E17            ! Speed of light (nm/s)
     param%eps_vacuum         = 1._wp                    ! Dielectric function of vacuum
     
     !-----------------------------------
     ! READ PARAMETERS FROM FILE
     !----------------------------------- 
     Call read_param(param)

     !-----------------------------------
     ! PROMPTS FOR SOME OF THE PARAMETERS
     !----------------------------------- 
     !     Call prompt_param(param)
     
     ! ---------------------------------
     ! --- Derived Quantities
     ! ---------------------------------
     
     ! Calculate the apparent island radius
     If (param%tr>=0._wp) Then
        param%Rapparent = param%R
     Else
        param%Rapparent = param%R * Sqrt(1._wp - param%tr**2)
     Endif
     
     ! Nature of the network and density
     Select case(param%network)
     Case('square')
        param%density         =  1._wp/(param%lattice_const**2)
     Case('hexagonal')
        param%density         =  1._wp/(sqrt(3._wp)/2._wp*param%lattice_const**2)
     Case('MFT','RPT')
        param%density         =  param%coverage/(pi*param%Rapparent**2)
        param%lattice_const	=  1._wp/(sqrt(param%density))
     Case Default
        Write(6,*) 'Error in the type of network : ',param%network, &
             ' not supported yet !'
        Pause
        Stop
     End Select
     
     
     ! Check for cluster overlapping
     If(2*param%Rapparent>param%lattice_const) Then
        Write(6,*) 'Error in the input : Clusters Overlapping !'
        Pause
     Endif
     
     ! Define the coverage, the aspect ration and the volume
     param%coverage = param%density*pi*param%Rapparent**2
     param%SR	  =  2*param%Rapparent/(1._wp+param%tr)/param%R 
     param%volume =  pi* param%R**3 *(2._wp/3._wp+param%tr-param%tr**3/3._wp)
     
     ! Notice that tr can be both positive and negative with the convention used
     If (param%tr >= param%MPpos) Then
        param%above = .True.
     Else
        param%above = .False.
     Endif
     
     ! Position of the substrate
     param%div_surface     =   param%tr*param%R
     
     ! m limits for integral and system solutions
     Select Case(param%interaction)
     Case('none','dipole')
        param%m = 1
     Case('quadrupole')
        param%m = 2
     Case Default 
        Write(6,*) 'Error in Initialize : ',param%interaction, &
             ' not supported yet !'
        Pause
        Stop
     End Select
     
     Return
   End Subroutine get_parameters
   
   !---------------------------------------------------------------
   Subroutine Read_Param(param)

     Implicit None
     Type(parameters)	::	param
     Logical	        ::	exi
     Integer		::	i	
     
     ! Read the input file for the various parameters
     Inquire(file='Sphere.inc',exist=exi)
     If(.not.exi) Then
        Write(6,*) 'Error in Read_Param ! File Sphere.inc non existing'
        Pause
        Stop
     Endif
     
     Open(10,file='Sphere.inc',status='unknown')
     ! Geometric parameters
     i = 1
     Read(10,*,err=20)	param%path
     i = i + 1
     Read(10,*,err=20)	param%geometry
     i = i + 1
     Read(10,*,err=20)	param%film_thickness1, param%film_thickness2
     i = i + 1
     Read(10,*,err=20)	param%R, param%tr, param%MPpos
     i = i + 1
     Read(10,*,err=20)	param%interaction, param%network, param%lattice_const, param%coverage
     
     ! Dielectric parameters
     i = i + 1
     Read(10,*,err=20)	param%out, param%polarization, param%theta0, param%fresnel
     i = i + 1
     Read(10,*,err=20)	param%energy_min, param%energy_max, param%Nenergy
     i = i + 1	
     Read(10,*,err=20)	param%island, param%substrate, param%coating
     i = i + 1		
     Read(10,*,err=20)	param%mean_free_path, param%surface_effects
     
     ! Calculation parameters
     i = i + 1
     Read(10,*,err=20)	param%polarizability, param%normalization, param%strenght
     i = i + 1
     Read(10,*,err=20)	param%bound, param%BC_energy, param%scaling
     i = i + 1
     Read(10,*,err=20)	param%Mpole_order
     i = i + 1	
     Read(10,*,err=20)  param%outfilename
     i = i + 1
     Read(10,*,err=20)	param%compa, param%expfilename
     
     
     Close(10,status='keep')
     Return
     
20   Write(6,*) 'Error during reading of Sphere.inc at line # ',i
     Pause
     Stop 
     
   End Subroutine Read_Param
   
   !---------------------------------------------------------------

   Subroutine prompt_param(param)
     
     Implicit None
     Type(parameters)   :: param
     
     !	  Write(6,'(A\)') ' Type of calculation     : '
     !	  Read(5,*) param%out
     !	  Write(6,'(A\)') ' Island material         : '
     !	  Read(5,*) param%island
     !      Write(6,'(A\)') ' Substrate material      : '
     !	  Read(5,*) param%substrate 
     !      Write(6,'(A\)') ' Geometry (island/film)  : '
     !	  Read(5,*) param%geometry 
     !	  Write(6,'(A\)') ' Temperature(K)          : '
     !	  Read(5,*) param%temperature
     
     Select case (trim(adjustl(param%geometry)))
     case ('film')
        Write(6,'(A\)') ' Mean film thickness     : '
        Read(5,*) param%film_thickness1   
     case('island')
        Write(6,'(A\)') ' Truncation ratio        : '
        Read(5,*) param%tr  
        Write(6,'(A\)') ' Sphere radius           : '
        Read(5,*) param%R   
        !  	     Write(6,'(A\)') ' Coverage                : '
        !	     Read(5,*) param%coverage     
        Write(6,'(A\)') ' Lattice constant        : '
        Read(5,*) param%lattice_const
        Write(6,'(A\)') ' # multipoles            : '
        Read(5,*) param%Mpole_order 
        !		 Write(6,'(A\)') ' Interaction             : '
        !		 Read(5,*) param%interaction
        !		 Write(6,'(A\)') ' Energy for BC           : '
        !		 Read(5,*) param%BC_energy       
        !  	     Write(6,'(A\)') ' Multipole position      : '
        !	     Read(5,*) param%MPpos 
        Write(6,'(A\)') ' Experimental file       : '
        Read(5,*) param%expfilename
     case default
        write(*,*) ' Geometry ',trim(adjustl(param%geometry)),' not supported ! '
        Pause
        Stop
     end Select
     
     Write(6,*)
     
     Return
   End Subroutine prompt_param
   
   !---------------------------------------------------------------
   
   Subroutine initialize(A,bd,bq,param,Fresnel_coeff, &
        mpoled,mpoleq,int,int_tr1,surf_const_coef,zeta)
     
     Use FMZM, only		: ZM_SET
     
     Implicit None
     Type(parameters)                     :: param
     Type(integrals),Pointer              :: Int(:,:,:), int_tr1(:,:,:)
     Type(surface_constitutive), Pointer  :: surf_const_coef(:)   
     Real(wp),Pointer                     :: zeta(:,:,:), Fresnel_coeff(:)
     Complex(wpc), Pointer                :: A(:,:)
     Complex(wpc), Pointer                :: bd(:,:),bq(:,:)
     Complex(wpc), Pointer                :: mpoled(:,:,:),mpoleq(:,:)
     Integer                              :: mpo
     Integer                              :: m
     
     Write(6,*) '*******************************************'
     Write(6,*) '                SPHERE'
     Write(6,*) ' Optics calculations for supported islands'
     Write(6,*) '              July 1998'
     Write(6,*) '*******************************************'
     Write(6,*)
     
     ! Initialize the parameters of the problem
     Call get_parameters(param)
    
     mpo = param%Mpole_order
     m	 = param%m
     
     !------------------------------------------------
     ! Allocate space for vectors and matrices
     !------------------------------------------------
     Select Case(trim(adjustl(param%geometry)))
     Case('film','2film','thin_cap')
        ! Allocations for the output
        Allocate( Fresnel_coeff(param%Nenergy) )
        Allocate( surf_const_coef(param%Nenergy)) 
     Case('island')
        ! Allocations for the output
        Allocate( Fresnel_coeff(param%Nenergy) )
        Allocate( surf_const_coef(param%Nenergy))            
        ! Allocate space for the linear (matrix) system
        Allocate( A(2*mpo,2*mpo) )
        Allocate( bd(2*mpo,0:1),bq(2*mpo,0:2) ) 
        ! Allocate space for the various types of integrals
        !  Notice : index 2=0 is only used for Q-type integrals
        Allocate( Int(0:mpo,0:mpo,0:m) )
        Allocate( int_tr1(0:mpo,0:mpo,0:m) )
        Allocate( zeta(0:mpo,0:mpo,0:m) )
        Allocate( mpoled(2,0:1,param%Nenergy) )
        Allocate( mpoleq(3,param%Nenergy) )
     End Select
     
     ! Initialize the dielectrics constants
     Call initialize_dielectric_constants(param)
     
     ! Calculate the zeta-coeffisients
     If(param%geometry=='island') Then 
        Call get_zeta(zeta,param)
     Endif
     
     ! Initialiaze the FMZM package
     Call ZM_SET(param%MP_precision)
     
     Return
     
   End Subroutine initialize
   
   !---------------------------------------------------------------
   
   Subroutine initialize_dielectric_constants(param)
     
     Implicit None
     Type(parameters)			::	param
     Real(wp)				    ::  denergy,de
     Integer						::	di,ienergy
     
     
     ! Allocate space for the dielectric constants
     Allocate(param%energy(param%Nenergy))
     Allocate(param%eps_substrate(param%Nenergy))   
     Allocate(param%eps_island(param%Nenergy))

     ! Fill the energy vector
     denergy =  (param%energy_max-param%energy_min)/(param%Nenergy-1)
     param%energy = arth(param%energy_min,denergy,param%Nenergy)
     
     ! Fills the dielectric constant vectors
     Call dielectric_constants(param%energy,param%eps_island,param%island,param%path)
     Call dielectric_constants(param%energy,param%eps_substrate,param%substrate,param%path)
     
     ! To take intoaccount the thermoreflectance
     If(param%thermoreflectance) Then
        Write(6,*)
        Write(6,'(a\)') ' Enter the energy shift(eV) for the substrate : '
        Read(5,*) de
        di = Int(de/denergy)
        Do ienergy=param%Nenergy,di+1,-1
	   param%eps_substrate(ienergy) = param%eps_substrate(ienergy-di)
        Enddo
     Endif
     
     ! Finite size scaling of the dielectric constant of the island 
     if (param%mean_free_path/='none')  &
          Call dielectric_func_corrections(param%eps_island,param%island,param%path,param)
     
     Return
     
   End Subroutine initialize_dielectric_constants
   
   !---------------------------------------------------------------
   
   Subroutine dielectric_constants(energy,epsilon,material,path)
     
     Implicit None
     Real(wp)			:: energy(:)
     Complex(wpc)               :: Epsilon(:)
     Character(len=*)		:: material,path
     Character(len=200)         :: filename
     Logical			:: exi
     Integer			:: lines,start,i
     Real(wp),    Allocatable	:: x(:)
     Complex(wpc), Allocatable  :: y(:)
     Real(wp)                   :: tmp(2),x1,x2,dx
     Integer			:: unit
     Complex(wpc)               :: slope
     Complex(wpc),parameter	:: im=(0._wp,1._wp)
     
     ! Opens the relevant data file for the given material
     ! Notice that the data-files contains the values for the 
     ! index of refracction, n,k extacted from the data base of
     ! SOPRA.
     ! Therefore "epsilon = epsilon**2" at the bottom of this routine    
     filename = Trim(Adjustl(path))//directory_separator//Trim(Adjustl(material))//'.nk'
     Inquire(file = filename, exist = exi)
     If(.not. exi) then
        Write(*,*) 'ERROR (get_epsilon) : File = ',trim(adjustl(filename)),' non existing'
        Pause
        Stop
     Endif
     Open(10,file=filename,status='old')
     
     !	Open(unit=10,file=filename)
     !	Read(10,*) lines
     !	Allocate( x(lines), y(lines) )
     !	Do i=1,lines
     !	 Read(10,*) x(i),x1,x2
     !	 y(i)=x1+im*x2
     !	Enddo
   
     Open(unit=10,file=filename)
     Read(10,*) unit,x1,x2,lines
     dx = (x2 - x1)/(lines-1)

     Allocate( x(lines), y(lines) )
     Select Case (unit)
     Case(1)
        ! Unit = Electron Volt (eV)
        Do i=1,lines 
           Read(10,*) tmp(1), tmp(2)
           x(i) = x1 + (i-1)*dx
           y(i) = tmp(1)+imu*tmp(2)
        Enddo
     Case(2)
        ! Unit = WaveLenght (*m)
        Do i=lines,1,-1 
           Read(10,*) tmp(1), tmp(2)
           x(i) = x1 + (lines-i)*dx
           y(i) = tmp(1)+imu*tmp(2)
        Enddo
        x(:) = 1.243/x(:) ! Conversiob *m-->eV
     End Select
     
     Close(10)

     ! Do the interpolation
     Do i=1,Size(energy,1)
        start=locate(Real(x(:),sp),Real(energy(i),sp))
        If ((start==0).Or.(start==lines)) then
           Write(*,*) 'ERROR (get_epsilon) : Energy not in range : ',energy(i)
           Pause
           stop
        End If
        ! Linear interpolation
        slope = (y(start+1)-y(start))/(x(start+1)-x(start))
        Epsilon(i) = y(start) + slope*(energy(i)-x(start))
        !		write(67,*) energy(i),real(epsilon(i)),aimag(epsilon(i))
     Enddo
     
     ! Calculates the dielectric constant (from the refraction index)
     epsilon = epsilon**2
     Deallocate(x,y)
     
     Return
   End Subroutine dielectric_constants
   
   !---------------------------------------------------------------
   
   Subroutine dielectric_func_corrections(epsilon,material,path,param,d)
     ! Calculates the correction to the dielectric functetion due to the 
     ! finit size of the islands
     ! See Physica A143, 164 (1987), for details 
     
     Implicit None
     Type(Parameters)          :: param
     Complex(wpc)              :: Epsilon(:),tmp_surf_eff
     Character(len=*)          :: material,path
     Character(len=200)        :: filename
     Logical                   :: exi
     Integer                   :: ienergy 
     Real(wp)                  :: omega_p,inv_relaxation_time,fermi_velocity
     Real(wp)                  :: R_eff,inv_tau_bulk,inv_tau,E,surf_eff,disp_T
     Real(wp)                  :: percent
     Real(wp),optional         :: d
     Real(wp)                  :: A
     
     
     ! Opens the relevant data file for the given material
     filename = Trim(Adjustl(path))//directory_separator//'Finite_Size'//directory_separator//Trim(Adjustl(material))//'.dat'
     Inquire(file = filename, exist = exi)
     If(.not. exi) then
        Write(*,*) 'ERROR (dielectric_func_corrections) : File = ', &
             filename,' non existing'
        Pause
        Return
     Endif 
     Open(10,file=filename,status='old')
     
     ! Reads the data (all quanteties are multiplied by hbar)
     Read(10,*) omega_p             ! Plasma frequency  (eV)
     Read(10,*) inv_relaxation_time ! Inverse bulk relaxation time (eV) 
     Read(10,*) fermi_velocity      ! Fermi velicity (eV nm)
     Read(10,*) surf_eff            ! Surface effects (eV^2) 
     Read(10,*) disp_T		    ! Dispersion of plasma frequency with T
     Close(10)

     inv_tau_bulk = inv_relaxation_time
     A	= 0.8_wp
     select case (trim(adjustl(param%geometry)))
     case('film','2film')
        R_eff        = param%film_thickness1
     case('island')
        R_eff        = (1+param%tr)*param%R          ! Physica A, Eq. (3.7)  
     end select
     
     ! Value of effective radius
     If(Present(d)) R_eff = d
     
     ! Temperature effects on the dielectric constant
     ! See Rocca Moresco Valbusa PRB 45,3 1399 (1992)
     omega_p = omega_p + disp_T * (param%temperature-300)
     
     ! Nature of mean free path limitation
     Select case(param%mean_free_path)
     Case('finite_size')
        inv_tau      = inv_tau_bulk + A*fermi_velocity/R_eff  ! Physica A, Eq. (3.6)
     Case('s-only')
        Write(6,'(a,f6.2,a\)') ' Enter the inverse of relaxation time (eV-1) (bulk= ',inv_tau_bulk,')  : '
        Read(5,*) inv_tau
     Case('tau')
        Write(6,'(a,f6.2,a\)') ' Enter the inverse of relaxation time (eV-1) (bulk= ',inv_tau_bulk,')  : '
        Read(5,*) inv_tau
     Case('manual')
        Write(6,'(a\)') ' Enter the percentage of bulk absorption : '
        Read(5,*) percent
        Write(6,*)
        inv_tau      = inv_tau_bulk + A*fermi_velocity/R_eff
     Case('A-parameter')
        Write(6,'(a\)') ' Enter the A-parameter : '
        Read(5,*) A
       inv_tau      =   inv_tau_bulk + A*fermi_velocity/R_eff
    Case default
       Write(6,*) 'Error in dielectric_func_corrections for param%mean_free_path'
    End select
    
    
    ! Update the dielectric constant of the island-material 
    ! See Physica A143, 164 (1987)   
    Do ienergy = 1,param%Nenergy
       E                 =    param%energy(ienergy) 
       ! Surface effects 
       ! See Liebsch, PRB B48, 11 317 (1993).
       ! Eq. (5.1) and (5.2) 
       if (param%surface_effects) then
          tmp_surf_eff = omega_p**2/(E**2 + surf_eff/R_eff + imu*E*inv_tau)
       else
          tmp_surf_eff = omega_p**2/(E**2 + imu*E*inv_tau)
        end if
        ! Dielectric constant of d-electrons for silver
        Epsilon(ienergy)  =  Epsilon(ienergy)                         &
             + omega_p**2/(E**2 + imu*E*inv_tau_bulk)
        ! Type of mean free path 
        select case(param%mean_free_path)
        case('s-only')
           Epsilon(ienergy)  =  Real(Epsilon(ienergy)) - tmp_surf_eff
        case('tau')
           Epsilon(ienergy)  =  Epsilon(ienergy) - tmp_surf_eff
        case('manual')
           Epsilon(ienergy)  =	 Real(Epsilon(ienergy)-tmp_surf_eff) + &
                imu*percent*Aimag(Epsilon(ienergy)-tmp_surf_eff)
        case('finite_size','A-parameter')
           Epsilon(ienergy)  =  Epsilon(ienergy) - tmp_surf_eff
        case default
           write(6,*) 'ERROR (dielectric_func_corrections) : Param%mean_free_path = ', &
                ' non existing'
        end select							            
     Enddo
     
   End Subroutine dielectric_func_corrections


   !------------------------------------------------------------------

   Subroutine get_zeta(zeta,param)
     
     Implicit None
     Type(parameters)       :: param
     Real(wp)               :: zeta(0:,0:,0:)
     Integer                :: l1,l2,m
     Real(wp)               :: factorial
     
     zeta(:,:,:) = 0._wp
     
     Do m=0,param%m
        Selectcase(m)
        Case(0)
           Do l1=0,param%Mpole_order
              Do l2=0,param%Mpole_order
                 zeta(l1,l2,m) = 0.5_wp *  Sqrt( (2*l1+1._wp)*(2*l2+1._wp) )
              Enddo
           Enddo
        Case(1)
           Do l1=1,param%Mpole_order
              Do l2=1,param%Mpole_order
                 factorial = 1._wp/( l1*(l1+1._wp)*l2*(l2+1._wp) )
                 zeta(l1,l2,m) = 0.5_wp *  Sqrt( (2*l1+1._wp)*(2*l2+1._wp)*factorial ) 
              Enddo
           Enddo
        Case(2)
           Do l1=2,param%Mpole_order
              Do l2=2,param%Mpole_order
                 factorial = 1._wp/( (l1-1._wp)*l1*(l1+1._wp)*(l1+2._wp)* &
                      (l2-1._wp)*l2*(l2+1._wp)*(l2+2._wp) )
                 zeta(l1,l2,m) = 0.5_wp *  Sqrt( (2*l1+1._wp)*(2*l2+1._wp)*factorial )
              Enddo
           Enddo
        End Select
     Enddo
     
     Return
     
   End Subroutine get_zeta

  !-------------------------------------------------------


End Module Initialize_mod
