
!----------------------------------------------------------------------
!              SPHERE part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: legendre.f90,v 1.1 2001/08/11 15:43:34 ingves Exp ingves $
!-----------------------------------------------------------------------


Module Legendre
  
  Use Global_definitions
  Implicit None

  ! Generic routine interface for fixed Gauss-Legendre integration
  Interface gausslegendre
     Module Procedure gauleg_sp, gauleg_dp
  End Interface

  ! Generic routine interface for Legendre polynomials for 
  ! vectors input
  Interface asslegpoly
     Module Procedure plgndr_v_sp,  plgndr_v_dp
  End Interface

  ! Generic routine interface for Legendre polynomials for 
  ! real
  Interface asslegpoly1
     Module Procedure plgndr_s_sp,  plgndr_s_dp
  End Interface

  Public :: gausslegendre, asslegpoly, asslegpoly1


Contains

  ! ----------------------------------------------------------
  ! --- Routines for Gauss-Legendre integration
  ! ----------------------------------------------------------
  
  ! Single precision version
  Subroutine gauleg_sp(x1,x2,x,w)
    Implicit None
    Real(SP), Intent(IN) :: x1,x2
    Real(SP), Dimension(:), Intent(OUT) :: x,w
    Real(DP), Parameter :: EPS=3.0e-14_dp
    Integer(I4B) :: its,j,m,n
    Integer(I4B), Parameter :: MAXIT=10
    Real(DP) :: xl,xm
    Real(DP), Dimension((Size(x)+1)/2) :: p1,p2,p3,pp,z,z1
    Logical(LGT), Dimension((Size(x)+1)/2) :: unfinished
    n=Size(x)
    m=(n+1)/2
    xm=0.5_dp*(x2+x1)
    xl=0.5_dp*(x2-x1)
    z=Cos(PI_D*(arth(1._dp,1._dp,m)-0.25_dp)/(n+0.5_dp))
    unfinished=.True.
    Do its=1,MAXIT
       Where (unfinished)
          p1=1.0
          p2=0.0
       End Where
       Do j=1,n
          Where (unfinished)
             p3=p2
             p2=p1
             p1=((2.0_dp*j-1.0_dp)*z*p2-(j-1.0_dp)*p3)/j
          End Where
       End Do
       Where (unfinished)
          pp=n*(z*p1-p2)/(z*z-1.0_dp)
          z1=z
          z=z1-p1/pp
          unfinished=(Abs(z-z1) > EPS)
       End Where
       If (.Not. Any(unfinished)) Exit
    End Do
    If (its == MAXIT+1) Then 
	 Write(6,* ) 'too many iterations in gauleg'
	 Pause
	 Stop
	Endif
    x(1:m)=xm-xl*z
    x(n:n-m+1:-1)=xm+xl*z
    w(1:m)=2.0_dp*xl/((1.0_dp-z**2)*pp**2)
    w(n:n-m+1:-1)=w(1:m)
  End Subroutine gauleg_sp



  ! Double precision version
  Subroutine gauleg_dp(x1,x2,x,w)
    Implicit None
    Real(DP), Intent(IN) :: x1,x2
    Real(DP), Dimension(:), Intent(OUT) :: x,w
    Real(DP), Parameter :: EPS=3.0e-14_dp
    Integer(I4B) :: its,j,m,n
    Integer(I4B), Parameter :: MAXIT=10
    Real(DP) :: xl,xm
    Real(DP), Dimension((Size(x)+1)/2) :: p1,p2,p3,pp,z,z1
    Logical(LGT), Dimension((Size(x)+1)/2) :: unfinished
    n=Size(x)
    m=(n+1)/2
    xm=0.5_dp*(x2+x1)
    xl=0.5_dp*(x2-x1)
    z=Cos(PI_D*(arth(1._dp,1._dp,m)-0.25_dp)/(n+0.5_dp))
    unfinished=.True.
    Do its=1,MAXIT
       Where (unfinished)
          p1=1.0
          p2=0.0
       End Where
       Do j=1,n
          Where (unfinished)
             p3=p2
             p2=p1
             p1=((2.0_dp*j-1.0_dp)*z*p2-(j-1.0_dp)*p3)/j
          End Where
       End Do
       Where (unfinished)
          pp=n*(z*p1-p2)/(z*z-1.0_dp)
          z1=z
          z=z1-p1/pp
          unfinished=(Abs(z-z1) > EPS)
       End Where
       If (.Not. Any(unfinished)) Exit
    End Do
    If (its == MAXIT+1) Then 
	 Write(6,* ) 'too many iterations in gauleg'
	 Pause
	 Stop
	Endif
    x(1:m)=xm-xl*z
    x(n:n-m+1:-1)=xm+xl*z
    w(1:m)=2.0_dp*xl/((1.0_dp-z**2)*pp**2)
    w(n:n-m+1:-1)=w(1:m)
  End Subroutine gauleg_dp

  
  ! ----------------------------------------------------------
  ! ---  Routines for the calculation of the Assosiate 
  ! ---  Legendre polynomials with vector input arguments
  ! ----------------------------------------------------------

  ! Single precision version
  Function plgndr_v_sp(l,m,x_sp)  Result(Plm_sp)
    
    Implicit None
    ! Global
    Integer(I4B), Intent(IN)               :: l,m
    Real(SP), Dimension(:), Intent(IN)     :: x_sp
    Real(SP), Dimension(Size(x_sp))        :: Plm_sp
    ! Local
    Integer                                :: i
    
    Do i=1,Size(x_sp)
       Plm_sp(i)  =  plgndr_s_sp(l,m,x_sp(i))	 
    Enddo
    
  End Function plgndr_v_sp



  ! Double precision version
  Function plgndr_v_dp(l,m,x_dp)     Result(Plm_dp)
    
    Implicit None
    ! Global
    Integer(I4B), Intent(IN)             :: l,m
    Real(DP), Dimension(:), Intent(IN)   :: x_dp
    Real(DP), Dimension(Size(x_dp))      :: Plm_dp
    ! Local
    Integer                              :: i
    
    Do i=1,Size(x_dp)
       Plm_dp(i)	=	plgndr_s_dp(l,m,x_dp(i))	 
    Enddo
    
  End Function plgndr_v_dp


  ! ----------------------------------------------------------
  ! ---  Routines for the calculation of the Assosiate 
  ! ---  Legendre polynomials with real input parameter
  ! ----------------------------------------------------------

  ! Single precision version
  Function plgndr_s_sp(l,m,x_sp)   Result(Plm_sp)
    
    Implicit None
    ! Global
    Integer(I4B), Intent(IN)   :: l,m
    Real(SP), Intent(IN)       :: x_sp
    Real(SP)                   :: Plm_sp
    ! Local
    Integer(I4B)               :: ll
    Real(SP)                   :: pll,pmm,pmmp1,somx2
    
    If(.not.(m >= 0.and.m <= l.and.abs(x_sp) <= 1.0)) Then
       Write(6,*) 'Error in plgndr_s arguments'
       Pause
       Stop
    Endif
    pmm=1.0_sp
    if (m > 0) then
       somx2=sqrt((1.0_sp-x_sp)*(1.0_sp+x_sp))
       pmm=product(arth(1.0_sp,2.0_sp,m))*somx2**m
       if (mod(m,2) == 1) pmm=-pmm
    end if
    if (l == m) then
       Plm_sp=pmm
    else
       pmmp1=x_sp*(2*m+1)*pmm
       if (l == m+1) then
          Plm_sp=pmmp1
       else
          do ll=m+2,l
             pll=(x_sp*(2*ll-1)*pmmp1-(ll+m-1)*pmm)/(ll-m)
             pmm=pmmp1
             pmmp1=pll
          end do
          Plm_sp=pll
       end if
    end if
    
  End Function plgndr_s_sp
  
  
  
  ! Double precision version
  Function plgndr_s_dp(l,m,x_dp)   Result(Plm_dp)
    
    Implicit None
    ! Global
    Integer(I4B), Intent(IN)   :: l,m
    Real(DP), Intent(IN)       :: x_dp
    Real(DP)                   :: Plm_dp
    ! Local
    Integer(I4B)               :: ll
    Real(DP)                   :: pll,pmm,pmmp1,somx2
    
    
    If(.not.(m >= 0.and.m <= l.and.abs(x_dp) <= 1.0)) Then
	 Write(6,*) 'Error in plgndr_s arguments'
	 Pause
	 Stop
      Endif
      pmm=1.0_dp
      if (m > 0) then
         somx2=sqrt((1.0_dp-x_dp)*(1.0_dp+x_dp))
         pmm=product(arth(1.0_dp,2.0_dp,m))*somx2**m
         if (mod(m,2) == 1) pmm=-pmm
      end if
      if (l == m) then
         Plm_dp=pmm
      else
         pmmp1=x_dp*(2*m+1)*pmm
         if (l == m+1) then
            Plm_dp=pmmp1
         else
            do ll=m+2,l
               pll=(x_dp*(2*ll-1)*pmmp1-(ll+m-1)*pmm)/(ll-m)
               pmm=pmmp1
               pmmp1=pll
            end do
            Plm_dp=pll
         end if
      end if
      
    End Function plgndr_s_dp
    
  End Module Legendre




