
!----------------------------------------------------------------------
!              SPHERE part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: optics_mod.f90,v 1.1 2001/08/11 15:44:56 ingves Exp ingves $
!-----------------------------------------------------------------------


Module Optics_mod

  Use Global_definitions
  Use Interaction_mod

Contains

  !---------------------------------------------------------------
  Subroutine surf_const_coef_film(surf_const_coef, param)
    ! Calculates the surface constitutive coefficients of a thin contiuous film
    ! SEE : Vlieger/Bedeaux : Thin solid Films 69, 107 (1980); Eq.(127)
    
    Implicit None
    Type(parameters)            :: param
    Type(surface_constitutive)  :: surf_const_coef(:)
    integer                     :: Ienergy
    complex(wpc)                :: e1,e3
    
    e1     =  param%eps_vacuum    ! Vacuum
    do ienergy = 1,param%Nenergy
       e3     = param%eps_island(ienergy)
       surf_const_coef(ienergy)%gamma  = (e3-e1) * param%film_thickness1
       surf_const_coef(ienergy)%beta   = (1._wp/e1 - 1._wp/e3) * param%film_thickness1
       surf_const_coef(ienergy)%tau    = param%film_thickness1**2*(e3-e1)/2
       surf_const_coef(ienergy)%delta  = param%film_thickness1**2*(e3/e1-e1/e3)/2
    enddo
    
  End Subroutine surf_const_coef_film
  
  !------------------------------------------------------------------------

  Subroutine surf_const_coef_2film(surf_const_coef, param)
    ! Calculates the surface constitutive coefficients of a 
    ! stacking of 2 thin contiuous films
    
    Use Initialize_mod, only : dielectric_constants,dielectric_func_corrections
    
    Implicit None
    Type(parameters)            :: param
    Type(surface_constitutive)  :: surf_const_coef(:)
    Type(surface_constitutive)  :: SCC1(param%Nenergy),SCC2(param%Nenergy)
    Complex(wpc)                :: eps_coating(param%Nenergy)
    Integer                     :: Ienergy
    Complex(wpc)                :: e1,e3
    Real(wp)                    :: t1,t2

    ! Some abreviations
    t1   =  param%film_thickness1
    t2   =  param%film_thickness2
    e1	 =  param%eps_vacuum    ! Vacuum
    
    ! Get the dielectric constant of the coating 2 and correct it
    Call dielectric_constants(param%energy,eps_coating,param%coating,param%path)
    !$IS!    If (param%mean_free_path<>'none') &
    If (param%mean_free_path/='none') &
         Call dielectric_func_corrections(eps_coating,param%coating, &
         param%path,param,t2)
    
    
    ! get the surface Constituitve coefficient for both layers
    Do ienergy = 1,param%Nenergy
       e3     = param%eps_island(ienergy)
       SCC1(ienergy)%gamma  = (e3-e1) * t1
       SCC1(ienergy)%beta   = (1._wp/e1 - 1._wp/e3) * t1
       SCC1(ienergy)%tau    = t1**2*(e3-e1)/2
       SCC1(ienergy)%delta  = t1**2*(e3/e1-e1/e3)/2
       e3                   = eps_coating(ienergy)
       SCC2(ienergy)%gamma  = (e3-e1) * t2
       SCC2(ienergy)%beta   = (1._wp/e1 - 1._wp/e3) * t2
       SCC2(ienergy)%tau    = t2**2*(e3-e1)/2
       SCC2(ienergy)%delta  = t2**2*(e3/e1-e1/e3)/2
    Enddo
    
    ! Add the layers (formulas 3.63 of Bedeaux's book)
    surf_const_coef(:)%gamma =  SCC1(:)%gamma + SCC2(:)%gamma
    surf_const_coef(:)%beta  =  SCC1(:)%beta  + SCC2(:)%beta
    surf_const_coef(:)%tau   =  SCC1(:)%tau   + SCC2(:)%tau
    surf_const_coef(:)%delta =  SCC1(:)%delta + SCC2(:)%delta + &
	 (SCC2(:)%gamma*SCC1(:)%beta-SCC1(:)%gamma*SCC2(:)%beta)/2._wp
    
  End Subroutine surf_const_coef_2film
  
  !---------------------------------------------------------------
  
  Subroutine surf_const_coef_cap(surf_const_coef, param)
    ! Calculates the surface constitutive coefficients of a thin 
    ! spherical cap 
    ! SEE : Vlieger/Bedeaux book Section 8.5
    
    Implicit None
    Type(parameters)            :: param
    Type(surface_constitutive)  :: surf_const_coef(:)
    Complex(wpc)                :: alphad(2,2,param%Nenergy)
    
    Write(6,*)
    Write(6,*) '-----------------------------------------------------------'
    Write(6,*) ' Warning (subroutine : surf_const_coef_cap) !'
    Write(6,*) ' The formulas for a thin cap are valid only for '
    Write(6,*) ' a small effective height tr~-1 !'
    Write(6,*) '-----------------------------------------------------------'
    
    ! Calculate the (dimensionless) surface constitutive coefficients for all energies
    select case(trim(adjustl(param%interaction)))
    case('none')
       ! Calculate the (dimensionless) polarizabilities (alpha) 
       ! for all energies 
       Call polarizabilities_cap(alphad,param)
       Call surf_const_coef_nointeract(surf_const_coef,alphad,param)
    case('dipole')
       ! Calculate the (dimensionless) polarizabilities (alpha) 
       ! for all energies in the dipolar situation
       Call polarizabilities_cap(alphad,param)
       Call surf_const_coef_dipole(surf_const_coef,alphad,param)
    case default
       Write(*,*) 'ERROR (surf_const_coef_cap): Parameter ', &
            Trim(Adjustl(param%interaction)),' not supported (yet)'
       Pause
       Stop
    End select
    
  End Subroutine surf_const_coef_cap

  !---------------------------------------------------------------

  Subroutine surf_const_coef_island(surf_const_coef,Ad,Aq,param)
    ! Calculated the surface constitutive coefficients for the island geometry
    ! Generic routine
    
    Implicit None                       
    Type(parameters)                :: param
    Type(surface_constitutive)      :: surf_const_coef(param%Nenergy)
    Complex(wpc)                    :: Ad(2,0:1,param%Nenergy)
    Complex(wpc)                    :: Aq(3,param%Nenergy)
    Complex(wpc)                    :: alphad(2,2,param%Nenergy)
    Complex(wpc)                    :: alphaq(3,param%Nenergy)
    Real(wp)                        :: c0,s0,exp0,sqr1,sqr2,e1
    
	 
    ! Ad : results of the calculations with a constant field
    ! Aq : results for the calculations for a quadrupolar field    
    
    ! Calculate the (dimensionless) surface constitutive coefficients for all energies
    select case(trim(adjustl(param%interaction)))
    case('none')
       ! Calculate the (dimensionless) polarizabilities (alpha) 
       ! for all energies 
       Call polarizabilities(alphad,Ad,param)
       Call surf_const_coef_nointeract(surf_const_coef,alphad,param)
    case('dipole')
       ! Calculate the (dimensionless) polarizabilities (alpha) 
       ! for all energies in the dipolar situation
       Call polarizabilities(alphad,Ad,param)
       Call surf_const_coef_dipole(surf_const_coef,alphad,param)
       ! Correct to first order the potential development
       e1       = param%eps_vacuum
       c0       = Cos(param%theta0_calc)
       s0       = Sin(param%theta0_calc)
       exp0     = Exp(-imu*param%phi0_calc)
       sqr1     = Sqrt(pi/3._wp)
       sqr2     = Sqrt(pi/5._wp)
       Ad(1,0,:) = alphad(1,2,:)*sqr1*c0/(2*pi*e1)
       Ad(2,0,:) = alphad(2,2,:)*sqr2*c0/(pi*e1)
       Ad(1,1,:) = -alphad(1,1,:)*Sqrt(2._wp)*sqr1*s0*exp0/(4*pi*e1)
       Ad(2,1,:) = -alphad(2,1,:)*Sqrt(6._wp)*sqr1*s0*exp0/(4*pi*e1)
    case('quadrupole')
       ! Calculate the (dimensionless) polarizabilities (alpha) 
       ! for all energies in the quadrupolar situation
       Call polarizabilities(alphad,Ad,param)
       Call polarizabilities_quadrupole(alphaq,Aq,param)
       Call surf_const_coef_quadrupole(surf_const_coef,alphad,alphaq,param)
    End select
    
  End Subroutine surf_const_coef_island

   
!------------------------------------------------------------------

  Subroutine polarizabilities(alpha,A,param)
    !-------------------------------------------------------
    !--- The polarizabilites (alpha) are calculated from 
    !--- (Bedeaux's book) Eq. (5.6.43) (cf. Eq. (8.2.29))
    !--- when the MP are above the subsrate and from
    !--- Eq. (8.3.7) when thay are below 
    !--- Notice that dimensionless quanities are used.
    !--- Here A is the (dimensionless) multipole coefficients
    !-------------------------------------------------------
    Implicit None
    Type(parameters) :: param       
    Complex(wpc)     :: alpha(2,2,param%Nenergy),A(2,0:1,param%Nenergy)
    Integer          :: ienergy
    Real(wp)         :: c0,s0,vol
    Complex(wpc)     :: e1,e2
    Complex(wpc)     :: exp0,A10,A11,A20,A21

    ! Some energy independent abbreviations
    e1    =  param%eps_vacuum
    c0    =  Cos(param%theta0_calc)
    s0    =  Sin(param%theta0_calc) 
    exp0  =  Exp(-imu*param%phi0_calc)
    
    If (param%above) Then
       !----------------------------------------
       !---  MP above the subsrate; Eq. (8.2.29)
       !----------------------------------------
       Do ienergy = 1, param%Nenergy
          ! Some energy dependent abbreviations
          A10 = A(1,0,ienergy)
          A11 = A(1,1,ienergy)
          A20 = A(2,0,ienergy)
          A21 = A(2,1,ienergy)
          ! Calculates the (dimensionless) polarizabilities 
          ! 1st index : 1 : dipole order
          !             2 : quadropole order
          ! 2nd index : 1 : parallel
          !             2 : perpendicular
          ! Dipole order 
          alpha(1,1,ienergy) = -4*pi*e1*A11/(Sqrt(2*pi/3)*s0*exp0)
          alpha(1,2,ienergy) =  2*pi*e1*A10/(Sqrt(pi/3)*c0)
          ! Quadropole order
          alpha(2,1,ienergy) = -4*pi*e1*A21/(Sqrt(6*pi/5)*s0*exp0)
          alpha(2,2,ienergy) =    pi*e1*A20/(Sqrt(pi/5)*c0)
          ! Output for the polarizability
          If(param%polarizability) Then
             Open(31,file='Polarizability.dat',status='unknown')
             If(param%normalization) Then
                vol = param%volume/(param%R**3)
             Else
                vol = 1._wp
             Endif
             Write(31,'(f10.5,5x,2(2f13.5,8x))') &
                  param%energy(ienergy),alpha(1,1,ienergy)/vol,alpha(1,2,ienergy)/vol
          Endif
       Enddo
    Else
       !----------------------------------------
       !---  MP below the subsrate; Eq. (8.3.7)
       !----------------------------------------
       Do ienergy = 1, param%Nenergy
          ! Some energy dependent abbreviations
          e2 = param%eps_substrate(ienergy)
          A10 = A(1,0,ienergy)
          A11 = A(1,1,ienergy)
          A20 = A(2,0,ienergy)
          A21 = A(2,1,ienergy)
          ! Calculates the (dimensionless) polarizabilities 
          ! 1st index : 1 : dipole order
          !             2 : quadropole order
          ! 2nd index : 1 : parallel
          !             2 : perpendicular
          ! Dipole order 
          alpha(1,1,ienergy) = -4*pi*e2*A11/(Sqrt(2*pi/3)*s0*exp0)
          alpha(1,2,ienergy) =  2*pi*e2*A10/((e1/e2)*Sqrt(pi/3)*c0)
          ! Quadropole order
          alpha(2,1,ienergy) = -4*pi*e2*A21/(Sqrt(6*pi/5)*s0*exp0)
          alpha(2,2,ienergy) =    pi*e2*A20/((e1/e2)*Sqrt(pi/5)*c0)
          ! Output for the polarizability
          If(param%polarizability) Then
             Open(31,file='Polarizability.dat',status='unknown')
             If(param%normalization) Then
                vol = param%volume/(param%R**3)
             Else
                vol = 1._wp
             Endif
             Write(31,'(f10.5,5x,2(2f13.5,8x))') &
                  param%energy(ienergy),alpha(1,1,ienergy)/vol,(e1/e2)**2*alpha(1,2,ienergy)/vol
          Endif
       Enddo
    Endif
    Close(31)
    Return
  End Subroutine polarizabilities

  !---------------------------------------------------------------
	
  Subroutine  polarizabilities_quadrupole(alpha,A,param)
    !-------------------------------------------------------
    !--- The polarizabilites (alpha) are calculated from 
    !--- (Bedeaux's book) Eq. (10.88) 
    !--- when the MP are above the subsrate and from
    !--- Eq. (10.88) modified by the coordinates transformations
    !--- x<->-x y<-->y z<-->-z e1<-->e2 e3<-->e4 when thay are below 
    !--- Notice that dimensionless quanities are used.
    !--- Here Aq is the (dimensionless) multipole coefficients
    !-------------------------------------------------------

    Implicit None
    Type(parameters)     ::  param
    Complex(wpc)         ::  alpha(3,param%Nenergy)
    Complex(wpc)         ::  A(3,param%Nenergy)
    Real(wp)             ::  sqr
    Complex(wpc)         ::  e1,e2,A20,A21,A22
    Integer              ::  ienergy
    
    ! Some energy independent abbreviations
    e1      = param%eps_vacuum
    sqr	    = sqrt(6._wp*pi/5._wp)
    
    If (param%above) Then
       Do ienergy=1,param%Nenergy
          e2  = param%eps_substrate(ienergy)
          A20	=	A(1,ienergy)
          A21	=	A(2,ienergy)
          A22	=	A(3,ienergy)
          !----------------------------------------
          !---  MP above the subsrate; Eq. (10.88)
          !----------------------------------------
          alpha(1,ienergy) = -pi*e1*A20/(sqr*sqrt(2._wp/3._wp) ) + &
               3*pi*e1*A22/(imu*sqr)
          alpha(2,ienergy) = -4*pi*e1*A21/sqr/(1._wp-imu) + &
               4*pi*e1*A22/(imu*sqr)
          alpha(3,ienergy) = -2*pi*e1*A22/(imu*sqr)
       Enddo
    Else
       Do ienergy=1,param%Nenergy
          ! Doubt on the conversion 
          !------------------------------------------------
          !---  MP below the subsrate; Eq. (10.88) modified
          !------------------------------------------------
          e2  = param%eps_substrate(ienergy)
          A20	=	A(1,ienergy)
          A21	=	A(2,ienergy)
          A22	=	A(3,ienergy)
          alpha(1,ienergy) = -pi*e2*A20/(sqr*sqrt(2._wp/3._wp) ) - &
               3*pi*e2*A22/(imu*sqr)
          alpha(2,ienergy) = -4*pi*e2*A21/sqr/(1._wp+imu) - &
               4*pi*e2*A22/(imu*sqr)
          alpha(3,ienergy) =  2*pi*e2*A22/(imu*sqr)
       Enddo
    Endif
    Return
    
  End Subroutine polarizabilities_quadrupole

  !---------------------------------------------------------------

  Subroutine polarizabilities_cap(alpha,param)
    ! Calculates the polarizabilities in the case of a thin spherical
    ! cap using the limited development 
    ! Formulas 8.104 of Bedeaux's book
    
    Implicit None
    Type(parameters)      ::  param
    Complex(wpc)          ::  alpha(2,2,param%Nenergy)
    Real(wp)              ::  h,volume,e1,vol		
    Complex(wpc)          ::  e2,e3
    Integer               ::  ienergy
    
    
    ! Some abreviations
    e1					=	param%eps_vacuum
    h					=	1._wp - Abs(param%tr)
    volume				=	pi*h**2*(1._wp-h/3._wp)
    ! Values necessary for keeping constant the order
    ! of development in the surface constitutive coefficient
    param%tr			=	1._wp
    param%MPpos			=	0._wp
    param%div_surface   =   param%tr*param%R
    
    Write(6,*)
    Write(6,'(a,f8.4)') ' Effective height : ',h
    
    Do ienergy = 1,param%Nenergy
       e2	= param%eps_substrate(ienergy)
       e3	= param%eps_island(ienergy)
       ! Calculates the (dimensionless) polarizabilities 
       ! 1st index : 1 : dipole order
       !             2 : quadropole order
       ! 2nd index : 1 : parallel
       !             2 : perpendicular
       ! Dipole order 
       alpha(1,1,ienergy) =  volume*(e3-e1)
       alpha(1,2,ienergy) =  volume*e2**2*(e3-e1)/(e1*e3)
       ! Quadropole order
       alpha(2,1,ienergy) =  -volume*(e3-e1)
       alpha(2,2,ienergy) =  -volume*e2**2*(e3-e1)/(e1*e3) 
       ! Output for the polarizability
       If(param%polarizability) Then
          Open(31,file='Polarizability.dat',status='unknown')
          If(param%normalization) Then
             vol = volume
          Else
             vol = 1._wp
          Endif
          Write(31,'(f10.5,5x,2(2f13.5,8x))') &
               param%energy(ienergy),alpha(1,1,ienergy)/vol,(e1/e2)**2*alpha(1,2,ienergy)/vol
       Endif
    Enddo
    
    
  End Subroutine polarizabilities_cap
  
  !---------------------------------------------------------------

  Subroutine surf_const_coef_nointeract(surf_const_coef,alpha,param)
    ! Calculate the surface costitutive coefficient from Eq. (8,2.30),
    ! when the MP are located above the subsrate and by Eq. (8.3.12)
    ! when they are located below.
    ! NOTE : All these formulae assumes LOW COVERAGE
    !        No inter-island interaction is taken into account
    Implicit None
    Type(parameters)           :: param
    Type(surface_constitutive) :: surf_const_coef(param%Nenergy)
    Complex(wpc)               :: alpha(2,2,param%Nenergy)
    Integer                    :: ienergy
    Real(wp)                   :: density,d
    Complex(wpc)               :: e1,e2

    ! Some energy independent abbreviations
    e1       = param%eps_vacuum      
    ! Dimensionless d and density
	! d= absolute distance between surface and the location of multipoles
    d        = Abs( param%tr-param%MPpos )
    density  = param%density * param%R**2

    If (param%above) Then
       ! MP above the subsrate; Eq. (8.2.30)
       Do ienergy = 1,param%Nenergy
          surf_const_coef(ienergy)%gamma  =   density*alpha(1,1,ienergy) 
          surf_const_coef(ienergy)%beta   =   density*alpha(1,2,ienergy)/(e1**2)   
          surf_const_coef(ienergy)%delta  = - density/e1 *                          &
                                 Sum(alpha(2,:,ienergy) - d*alpha(1,:,ienergy))
          surf_const_coef(ienergy)%tau    = - density *                             & 
                                 ( alpha(2,1,ienergy) - d*alpha(1,1,ienergy) )   
       Enddo
    Else
       ! MP below the subsrate; Eq. (8.3.12)
       Do ienergy = 1,param%Nenergy
          e2 = param%eps_substrate(ienergy)
          surf_const_coef(ienergy)%gamma  =   density*alpha(1,1,ienergy) 
          surf_const_coef(ienergy)%beta   =   density*alpha(1,2,ienergy)/(e2**2)   
          surf_const_coef(ienergy)%delta  = - density/e2 *                          &
                                 Sum(alpha(2,:,ienergy) + d*alpha(1,:,ienergy))
          surf_const_coef(ienergy)%tau    = - density * (e1/e2) *                   & 
                                 ( alpha(2,1,ienergy) + d*alpha(1,1,ienergy) )
       Enddo
    Endif
    Return
  End Subroutine surf_const_coef_nointeract

  !---------------------------------------------------------------

  Subroutine surf_const_coef_dipole(surf_const_coef,alpha,param)
    ! Calculate the surface costitutive coefficient
    ! NOTE : All these formulae includes dipole interatction between the 
    !        various islands

    Implicit None
    Type(parameters)           :: param
    Type(surface_constitutive) :: surf_const_coef(param%Nenergy)
    Complex(wpc)               :: alpha(2,2,param%Nenergy)
    Integer                    :: ienergy
    Real(wp)                   :: S_mp,S_imp,L,factor,density,d,vol
    Complex(wpc)               :: e1,e2,eps
    Complex(wp)                :: paral,perp
    Real(wp)                   :: sqr

    ! Some energy independent abbreviations
    e1       = param%eps_vacuum
    sqr      = sqrt(4._wp*pi/5._wp)
    ! Dimensionless density,d,L
    ! d= absolute distance between surface and the location of multipoles
    d        = Abs( param%tr-param%MPpos )
    density  = param%density*param%R**2
    L        = param%lattice_const / param%R
    ! Special value for the dimensionlesslattice constant
    ! (P269 Bedeaux's book) for a random network
    if(param%network=='MFT'.or.param%network=='RPT') &
         L	=	1._wp/sqrt(density)  
    ! Lattice sums for the multipoles (MP) and image multipoles (IMP)
    S_mp     = lattice_sum(0._wp,2,param)
    S_imp    = lattice_sum(d,2,param )
    
    ! Renormalization of the polarisabilities in the Barrera model
    ! Phys. Rev. B 43 (17) P13819 (1991)
    If(param%network=='RPT') then
       Call Renorm_Polarizability(d,param,alpha(1,:,:)) 	
    Endif
      

    If (param%above) Then
       
       !------------------------------------------------
       !- MP above the substrate Eqs. 10.29 - 10.28
       !------------------------------------------------
       Do ienergy = 1,param%Nenergy
          e2  =  param%eps_substrate(ienergy)    
          eps =  (e1-e2)/(e2+e1)
          ! Uses the formulas 5-67 for conversion betwenn various polarizbilities
          ! Parallel component (10-29)
          factor = S_mp + eps*S_imp
          paral  = alpha(1,1,ienergy)/(4*pi*e1)
          paral  =  4*pi*e1*paral/(1._wp + paral*sqr/(L**3)*factor)
          ! Perpendicular component (10-28)
          factor = S_mp - eps*S_imp
          perp   = alpha(1,2,ienergy)/(4*pi*e1)
          perp   = 4*pi*e1*perp / (1._wp - 2._wp*perp*sqr/(L**3)*factor)
          ! Output for the polarizability
          If(param%polarizability) Then
             If(param%normalization) Then
                vol = param%volume/(param%R**3)
             Else
                vol = 1._wp
             Endif
             Open(31,file='Polarizability_Int.dat',status='unknown')
             Write(31,'(f10.5,5x,2(2f13.5,8x))') &
                  param%energy(ienergy),paral/vol,perp/vol
          Endif
          ! Fills the surface suceptibilities (10-30)
          surf_const_coef(ienergy)%beta   =  density*perp/(e1**2)
          surf_const_coef(ienergy)%gamma  =  density*paral
          surf_const_coef(ienergy)%delta  =  density*d*(paral + perp)/e1
          surf_const_coef(ienergy)%tau    =  density*d*paral
       Enddo
       
    Else
       
       !------------------------------------------------
       !- MP below the substrate Eqs. 10-33 10-34
       !------------------------------------------------
       Do ienergy = 1,param%Nenergy
          e2  =  param%eps_substrate(ienergy)    
          eps = (e2-e1)/(e2+e1)
          ! Uses the formulas (5-67) for conversion betwenn various polarizbilities
          ! Parallel component (10-34)
          factor =  S_mp + eps*S_imp
          paral	 =  alpha(1,1,ienergy)/(4*pi*e2)
          paral  =  4*pi*e2*paral/(1._wp + paral*sqr/(L**3)*factor)
          ! Perpendicular component (10-33)
          factor = S_mp - eps*S_imp
          perp = alpha(1,2,ienergy)/(4*pi*e2)
          perp = 4*pi*e2*perp / (1._wp - 2._wp*perp*sqr/(L**3)*factor)
          ! Output for the polarizability
          If(param%polarizability) Then
             If(param%normalization) Then
                vol = param%volume/(param%R**3)
             Else
                vol = 1._wp
             Endif
             Open(31,file='Polarizability_Int.dat',status='unknown')
             Write(31,'(f10.5,5x,2(2f13.5,8x))') &
                  param%energy(ienergy),paral/vol,perp/vol
          Endif
          ! Fills the surface suceptebilities (10-30)
          surf_const_coef(ienergy)%beta   =  density*perp/(e2**2)
          surf_const_coef(ienergy)%gamma  =  density*paral
          surf_const_coef(ienergy)%delta  =  -density*d*(paral + perp)/e2
          surf_const_coef(ienergy)%tau    =  -density*d*paral*e1/e2
       Enddo
       
    Endif
    Return
    
  End Subroutine surf_const_coef_dipole
  
  !---------------------------------------------------------------
	
  Subroutine surf_const_coef_quadrupole(surf_const_coef,alphad,alphaq,param)
    ! Calculate the surface costitutive coefficients 
    ! NOTE : All these formulae include quadrupole interactions between the 
    !        various islands
    ! Formulas 10-36 --> 10.55

    Implicit None
    Type(parameters)                 ::	param
    Type(surface_constitutive)       ::	surf_const_coef(param%Nenergy)
    Complex(wpc)                     ::	alphad(2,2,param%Nenergy)
    Complex(wpc)                     ::	alphaq(3,param%Nenergy) 
    Complex(wpc)                     ::	A1010,A1111,A2111,A2010,A2121,A2020,A2222,A1020,A1121
    Complex(wpc)                     ::	Az,Az10,Ap,Ap10	
    Complex(wpc)                     ::	e1,e2,eps,Dz,Dp
    Integer                          ::	ienergy
    Real(wp)                         ::	S2_mp,S2_imp,S3_mp,S3_imp,S4_mp,S4_imp
    Complex(wpc)                     ::	S2p,S3p,S4p,S2m,S3m,S4m
    Real(wp)                         ::	sq1,sq2,sq3,sq4,sq5,sq6
    Real(wp)                         ::	L,density,d
    
    ! Some energy independent abbreviations
    e1	=  param%eps_vacuum
    sq1	=  sqrt(15._wp*pi/7._wp)
    sq2	=  sqrt(pi)
    sq3	=  sqrt(pi/5._wp)
    sq4	=  sqrt(3._wp*pi/35._wp)
    sq5	=  sqrt(pi/35._wp)
    sq6	=  sqrt(5._wp*pi/7._wp)
    
    ! Dimensionless density,d,L
    ! d= absolute distance between surface and the location of multipoles
    d        = Abs( param%tr-param%MPpos ) 
    density  = param%density*param%R**2
    L        = param%lattice_const / param%R
    ! Special value for the dimensionlesslattice constant
    ! (P269 Bedeaux's book) for a random network
    if(param%network=='MFT'.or.param%network=='RPT') &
         L	=	1._wp/sqrt(density)
    
    ! Evaluation of the lattice sums
    S2_mp     = lattice_sum(0._wp,2,param)
    S2_imp    = lattice_sum(d,2,param )
    S3_mp     = lattice_sum(0._wp,3,param)
    S3_imp    = lattice_sum(d,3,param )
    S4_mp     = lattice_sum(0._wp,4,param)
    S4_imp    = lattice_sum(d,4,param )
    
    If (param%above) Then
       !------------------------
       !- MP above the substrate 
       !------------------------	
       Do ienergy = 1,param%Nenergy
          !Some abreviations
          e2  =	param%eps_substrate(ienergy)    
          eps =	(e1-e2)/(e2+e1)
          S2p	=	S2_mp + eps*S2_imp
          S2m	=	S2_mp - eps*S2_imp	
          S3p	=	S3_mp + eps*S3_imp
          S3m	=	S3_mp - eps*S3_imp
          S4p	=	S4_mp + eps*S4_imp
          S4m	=	S4_mp - eps*S4_imp	
          
          ! Polarizabilities of clusters interacting with the substrate
          ! Eqs. (5-67) Bedeaux's book
          A1010	=	alphad(1,2,ienergy)/(4*pi*e1)
          A1111	=	alphad(1,1,ienergy)/(4*pi*e1)
          A2111	=	3*alphad(2,1,ienergy)/(4*pi*e1*sqrt(5._wp))
          A2010	=	alphad(2,2,ienergy)/(2*pi*e1*sqrt(5._wp/3._wp))
          A2121	=	3*(alphaq(2,ienergy)+2*alphaq(3,ienergy))/(4*pi*e1)
          A2020	=	(2*alphaq(1,ienergy)+3*alphaq(3,ienergy))/(2*pi*e1)
          A2222	=	3*alphaq(3,ienergy)/(2*pi*e1)
          A1020	=	5._wp/3._wp*A2010
          A1121	=	5._wp/3._wp*A2111
          
          ! Equations 10-42 10-45
          Dz = (1._wp-4*A1010*S2m/(L**3)*sq3 - 6*A1020*S3m/(L**4)*sq4)* &
               (1._wp+2*A2010*S3p/(L**4)*sq1 + 4*A2020*S4p/(L**5)*sq2)+ &
               (4*A2010*S2m/(L**3)*sq3 + 6*A2020*S3m/(L**4)*sq4)* &
               (2*A1010*S3p/(L**4)*sq1 + 4*A1020*S4p/(L**5)*sq2)
          
          Dp = (1._wp+2*A1111*S2p/(L**3)*sq3 + 6*A1121*S3p/(L**4)*sq5)* &
               (1._wp-2*A2111*S3m/(L**4)*sq6 - 8*A2121*S4m/(L**5)*sq2/3)+ &
               (2*A2111*S2p/(L**3)*sq3 + 6*A2121*S3p/(L**4)*sq5)* &
               (2*A1111*S3m/(L**4)*sq6 + 8*A1121*S4m/(L**5)*sq2/3)
          
          ! Equations 10-41 10-42
          Az		=	4*pi*e1/(Dz)* &
               ( A1010*(1._wp+2*A2010*S3p/(L**4)*sq1+4*A2020*S4p/(L**5)*sq2) - &
               A2010*(2*A1010*S3p/(L**4)*sq1+4*A1020*S4p/(L**5)*sq2) ) 
          
          Az10	=	2*pi*e1*sqrt(5._wp/3._wp)/(Dz)* &
               ( A2010*(1._wp-4*A1010*S2m/(L**3)*sq3-6*A1020*S3m/(L**4)*sq4) + &
               A1010*(4*A2010*S2m/(L**3)*sq3+6*A2020*S3m/(L**4)*sq4) )
          
          Ap		=	4*pi/(Dp)* &
               ( A1111*(1._wp-2*A2111*S3m/(L**4)*sq6-8*A2121*S4m/(L**5)*sq2/3) + &
               A2111*(2*A1111*S3m/(L**4)*sq6+8*A1121*S4m/(L**5)*sq2/3) )
          
          Ap10	=	4*pi*e1*sqrt(5._wp)/3/(Dp)* &
               ( A2111*(1._wp+2*A1111*S2p/(L**3)*sq3+6*A1121*S3p/(L**4)*sq5) - &
               A1111*(2*A2111*S2p/(L**3)*sq3+6*A2121*S3p/(L**4)*sq5) )
          
          ! Fills the surface suceptebilities (10-47)
          surf_const_coef(ienergy)%beta   =  density*Az/(e1**2)
          surf_const_coef(ienergy)%gamma  =  density*Ap
          surf_const_coef(ienergy)%delta  =  -density*(Az10+Ap10-d*Az-d*Ap)/e1
          surf_const_coef(ienergy)%tau    =  -density*(Ap10-d*Ap)
          
       Enddo
       
    Else
       !------------------------
       !- MP below the substrate 
       !------------------------	
       Do ienergy = 1,param%Nenergy
          !Some abreviations
          e2  =	param%eps_substrate(ienergy)    
          eps =	(e1-e2)/(e2+e1)
          S2p	=	S2_mp + eps*S2_imp
          S2m	=	S2_mp - eps*S2_imp	
          S3p	=	S3_mp + eps*S3_imp
          S3m	=	S3_mp - eps*S3_imp
          S4p	=	S4_mp + eps*S4_imp
          S4m	=	S4_mp - eps*S4_imp	
          
          ! Polarizabilities of clusters interacting with the substrate
          ! Eqs. (5-67) Bedeaux's book
          A1010	=	alphad(1,2,ienergy)/(4*pi*e2)
          A1111	=	alphad(1,1,ienergy)/(4*pi*e2)
          A2111	=	3*alphad(2,1,ienergy)/(4*pi*e2*sqrt(5._wp))
          A2010	=	alphad(2,2,ienergy)/(2*pi*e2*sqrt(5._wp/3._wp))
          A2121	=	3*(alphaq(2,ienergy)+2*alphaq(3,ienergy))/(4*pi*e2)
          A2020	=	(2*alphaq(1,ienergy)+3*alphaq(3,ienergy))/(2*pi*e2)
          A2222	=	3*alphaq(3,ienergy)/(2*pi*e2)
          
          ! Equations 10-42 10-45
          Dz = (1._wp-4*A1010*S2p/(L**3)*sq3 - 6*A1020*S3p/(L**4)*sq4)* &
               (1._wp+2*A2010*S3m/(L**4)*sq1 + 4*A2020*S4m/(L**5)*sq2)+ &
               (4*A2010*S2p/(L**3)*sq3 + 6*A2020*S3p/(L**4)*sq4)* &
               (2*A1010*S3m/(L**4)*sq1 + 4*A1020*S4m/(L**5)*sq2)
          
          Dp = (1._wp+2*A1111*S2m/(L**3)*sq3 + 6*A1121*S3m/(L**4)*sq5)* &
               (1._wp-2*A2111*S3p/(L**4)*sq6 - 8*A2121*S4p/(L**5)*sq2/3)+ &
               (2*A2111*S2m/(L**3)*sq3 + 6*A2121*S3m/(L**4)*sq5)* &
               (2*A1111*S3p/(L**4)*sq6 + 8*A1121*S4p/(L**5)*sq2/3)
          
          ! Equations 10-41 10-42
          Az		=	4*pi*e2/(Dz)* &
               ( A1010*(1._wp+2*A2010*S3m/(L**4)*sq1+4*A2020*S4m/(L**5)*sq2) - &
               A2010*(2*A1010*S3m/(L**4)*sq1+4*A1020*S4m/(L**5)*sq2) ) 
          
          Az10	=	2*pi*e2*sqrt(5._wp/3._wp)/(Dz)* &
               ( A2010*(1._wp-4*A1010*S2p/(L**3)*sq3-6*A1020*S3p/(L**4)*sq4) + &
               A1010*(4*A2010*S2p/(L**3)*sq3+6*A2020*S3p/(L**4)*sq4) )
          
          Ap		=	4*pi*e2/(Dp)* &
               ( A1111*(1._wp-2*A2111*S3p/(L**4)*sq6-8*A2121*S4p/(L**5)*sq2/3) + &
               A2111*(2*A1111*S3p/(L**4)*sq6+8*A1121*S4p/(L**5)*sq2/3) )
          
          Ap10	=	4*pi*e2*sqrt(5._wp)/3/(Dp)* &
               ( A2111*(1._wp+2*A1111*S2m/(L**3)*sq3+6*A1121*S3m/(L**4)*sq5) - &
               A1111*(2*A2111*S2m/(L**3)*sq3+6*A2121*S3m/(L**4)*sq5) )
          
          ! Fills the surface suceptebilities (10-47)
          surf_const_coef(ienergy)%beta   =  density*Az/(e2**2)
          surf_const_coef(ienergy)%gamma  =  density*Ap
          surf_const_coef(ienergy)%delta  =  -density*(Az10+Ap10+d*Az+d*Ap)/e2
          surf_const_coef(ienergy)%tau    =  -density*(Ap10+d*Ap)*(e1/e2)
          
       Enddo
       
    Endif
    
    Return
    
    
  End Subroutine  surf_const_coef_quadrupole
  
  !--------------------------------------------------------------

  Subroutine Fresnel_calc(Fresnel_coeff,surf_const_coef,param)
    Implicit None     
    Type(parameters)                :: param
    Type(surface_constitutive)      :: surf_const_coef(param%Nenergy)
    Real(wp)                        :: Fresnel_coeff(param%Nenergy)
    Type(Invariants),Allocatable    :: Invar(:)
    Real(wp),allocatable            :: tmp(:)
    
    
    Selectcase(trim(adjustl(param%fresnel)))
       
    Case('constitutive')
       ! ------------------------------------------------------
       ! --- Surface constitutive coefficients
       ! ------------------------------------------------------
       Select Case(param%out)
       Case('DR','R')
          ! Calculate the differential reflectivity or reflectivity
          Call diff_ref_coef_constitutive(Fresnel_coeff,surf_const_coef,param)
       Case('DT','T')
          ! Calculate the differential transmittance or trasmittance
          Call diff_tran_coef_constitutive(Fresnel_coeff,surf_const_coef,param)
       Case('A')
          ! Calculate the absolute coefficient of absorption
          Allocate(tmp(param%Nenergy))
          param%out = 'R'
          Call diff_ref_coef_constitutive(Fresnel_coeff,surf_const_coef,param)
          tmp(:) = Fresnel_coeff(:)
          param%out = 'T'
          Call diff_tran_coef_constitutive(Fresnel_coeff,surf_const_coef,param)
          Fresnel_coeff(:) = 1._wp - tmp(:) - Fresnel_coeff(:)
          param%out = 'A'
          Deallocate(tmp)
       Case Default
          Write(6,*) 'Error in diff_ref_coef for param%out :',param%out
       End Select
       
    Case('constitutive_all')
       ! -------------------------------------------------------------
       ! --- Surface constitutive coefficients with second order terms
       ! -------------------------------------------------------------
       Select Case(param%out)
       Case('DR','R')
          ! Calculate the differential reflectivity or reflectivity
          Call diff_ref_coef_constitutive_all(Fresnel_coeff,surf_const_coef,param)
       Case('DT','T')
          ! Calculate the differential transmittance or trasmittance
          Call diff_tran_coef_constitutive_all(Fresnel_coeff,surf_const_coef,param)
       Case('A')
          ! Calculate the absolute coefficient of absorption
          Allocate(tmp(param%Nenergy))
          param%out = 'R'
          Call diff_ref_coef_constitutive_all(Fresnel_coeff,surf_const_coef,param)
          tmp(:) = Fresnel_coeff(:)
          param%out = 'T'
          Call diff_tran_coef_constitutive_all(Fresnel_coeff,surf_const_coef,param)
          Fresnel_coeff(:) = 1._wp - tmp(:) - Fresnel_coeff(:)
          param%out = 'A'
          Deallocate(tmp)
       Case Default
          Write(6,*) 'Error in diff_ref_coef for param%out :',param%out
       End Select
       
    Case('invariants')
       ! ------------------------------------------------------
       ! --- Surface Invariants
       ! ------------------------------------------------------
       ! Calculate the (dimensionless) invariants (invar) for all energies     
       Allocate(Invar(param%Nenergy))
       Call Invariants_calc(Invar,surf_const_coef,param)
       Select Case(param%out)
       Case('DR','R')
          ! Calculate the differential reflectivity or reflectivity      
          Call diff_ref_coef_invariants(Fresnel_coeff,Invar,param)
       Case('DT','T')
          ! Calculate the differential transmittance or trasmittance
          Call diff_tran_coef_invariants(Fresnel_coeff,Invar,param)
       Case('A')
          ! Calculate the absolute coefficient of absorption
          Call absorp_coef_invariants(Fresnel_coeff,Invar,param)
       Case Default
          Write(6,*) 'Error in diff_ref_coef for param%out :',param%out
       End Select
       Deallocate(Invar)
       
    Case('aspnes')
       ! ------------------------------------------------------
       ! --- Mac Intyre and Aspnes formulae modified by Bagchi
       ! --- cf YB,FA Thin Solid Film 125 (1985) 129   formulae 6-7
       ! ------------------------------------------------------
       Write(6,*)
       Write(6,*) '-----------------------------------------------------------'
       Write(6,*) ' Warning (subroutine : diff_ref_coef_aspnes) !'
       Write(6,*) ' Apsnes formulas only valid for differential reflectivity !'
       Write(6,*) '-----------------------------------------------------------'
       Call diff_ref_coef_aspnes(Fresnel_coeff,surf_const_coef,param)
       
    Case default
       Write(*,*) 'ERROR (diff_ref_coef): Parameter ', &
            Trim(Adjustl(param%fresnel)),' not supported (yet)'
       Pause
       Stop
    End Select
    
    Return
  End Subroutine Fresnel_calc

  !---------------------------------------------------------------
 
   Subroutine diff_ref_coef_constitutive(Fresnel_coeff,surf_const_coef,param)
     ! Calculates the differential reflectivity from knowledge of the
     ! surface constitutive coefficients.
     ! This method is described in detail in Haarmans and Bedeaux,
     ! Thin Solid Films 224, 117 (1992).
     ! See Eq. (34)--(37)
     Implicit None
     Type(parameters)            :: param
     Type(surface_constitutive)  :: surf_const_coef(:), chi
     Real(wp)                    :: Fresnel_coeff(:)
     Integer                     :: ienergy
     Real(wp)                    :: c0,s0,ooc
     Complex(wpc)                :: e1,n1,e2,n2,st,ct,Reflec,fresnel
     Complex(wpc)                :: factor(2),tmp(4)

     ! Some energy independent parameters
     e1 = Real(param%eps_vacuum)
     n1 = Sqrt(e1)
     c0 = Cos(param%theta0)
     s0 = Sin(param%theta0)

     Do ienergy = 1,param%Nenergy
        ! Omega over c (dimensionless)
        ooc            =  param%R * param%energy(ienergy) / (param%hbar*param%c)
        chi            =  surf_const_coef(ienergy)
        e2             =  param%eps_substrate(ienergy)
        n2             =  Sqrt(e2) 
        st             =  n1/n2*s0
        ct             =  Sqrt(1._wp - st**2) 
        
        Select case (trim(adjustl(param%polarization)))
        Case('p')
           tmp(1)         =  imu*ooc                                              
           tmp(2)         =  -tmp(1)**2/4._wp*e1*chi%beta*chi%gamma*s0**2         
           tmp(3)         =  chi%gamma*c0*ct + n1*n2*chi%beta*e1*s0**2            
           tmp(4)         =  chi%gamma*c0*ct - n1*n2*chi%beta*e1*s0**2    
           factor         =  (/ n2*c0 + n1*ct , n2*c0 - n1*ct /) 
           fresnel        =  factor(2)/factor(1)  
           Reflec         =  (factor(2) - tmp(1)*tmp(4) -tmp(2)*factor(2))/   &
                (factor(1) -tmp(1)*tmp(3) -tmp(2)*factor(1))
        Case('s')
           tmp(1)         =  imu*ooc 
           factor         =  (/ n1*c0 + n2*ct , n1*c0 - n2*ct /) 
           fresnel        =  factor(2)/factor(1) 
           Reflec         =  ( factor(2) + tmp(1)*chi%gamma )   /   &
                ( factor(1) - tmp(1)*chi%gamma )  
        End select
        
        Select case(param%out)
        Case('DR')
           ! The differential Reflection 
           Fresnel_coeff(ienergy)  = (Abs(Reflec/fresnel))**2-1._wp
        Case('R')
           ! Absolute coefficient
           Fresnel_coeff(ienergy)  = (Abs(Reflec))**2
        End select
        
     End Do
     Return
     
   End Subroutine diff_ref_coef_constitutive
   
   
   !---------------------------------------------------------------
   
   Subroutine diff_tran_coef_constitutive(Fresnel_coeff,surf_const_coef,param)
     ! Calculates the differential Transmittance from knowledge of the
     ! surface constitutive coefficients.
     ! This method is described in detail in Haarmans and Bedeaux,
     ! Thin Solid Films 224, 117 (1992).
     ! See Eq. (34)--(37)
     ! or in the parper of Bedeaux Physica 67 (1973) 55-73
     
     Implicit None
     Type(parameters)            :: param
     Type(surface_constitutive)  :: surf_const_coef(:), chi
     Real(wp)                    :: Fresnel_coeff(:)
     Integer                     :: ienergy
     Real(wp)                    :: c0,s0,ooc
     Complex(wpc)                :: e1,n1,e2,n2,st,ct
     Complex(wpc)                :: Transmi,fresnel
     Complex(wpc)                :: factor,tmp(3)
     
     Write(6,*)
     Write(6,*) '----------------------------------------------------'
     Write(6,*) ' Warning (subroutine : diff_tran_coef_constitutive) !'
     Write(6,*) ' The transmission coefficient is only valid for'
     Write(6,*) ' non-absorbing substrate above the total reflection'
     Write(6,*) '----------------------------------------------------'
     
     ! Some energy independent parameters
     e1 = Real(param%eps_vacuum)
     n1 = Sqrt(e1)
     c0 = Cos(param%theta0)
     s0 = Sin(param%theta0)
     
     Do ienergy = 1,param%Nenergy
        ! Omega over c (dimensionless)
        ooc            =  param%R * param%energy(ienergy) / (param%hbar*param%c)
        chi            =  surf_const_coef(ienergy)
        e2             =  param%eps_substrate(ienergy)
        n2             =  Sqrt(e2) 
        st             =  n1/n2*s0
        If(Real(st)>1._wp) Then
           Write(6,*) 'Total reflection at : ',param%energy(ienergy),' eV !'
           Write(6,*) '(subroutine : diff_tran_coef_constitutive)'
        Endif
        ct             =  Sqrt(1._wp - st**2) 
        tmp(1)         =  imu*ooc                                              
        tmp(2)         =  -tmp(1)**2/4._wp*e1*chi%beta*chi%gamma*s0**2         
        tmp(3)         =  chi%gamma*c0*ct + n1*n2*chi%beta*e1*s0**2    
        
        select case (trim(adjustl(param%polarization)))
        case('p')
           factor         =  n2*c0 + n1*ct 
           fresnel        =  2*n1*c0 / factor  
           Transmi         =  2*n1*c0* (1+tmp(2)) /   &
                (factor -tmp(1)*tmp(3) -tmp(2)*factor)
        case('s')
           factor         =  n1*c0 + n2*ct 
           fresnel        =  2*n1*c0 / factor 
           Transmi         =  2*n1*c0   /   &
                ( factor - tmp(1)*chi%gamma )  
        end select
        
        select case(param%out)
        case('DT')
           ! The differential Transmission 
           Fresnel_coeff(ienergy)  = Abs(Transmi/fresnel)**2-1._wp
        case('T')
           ! The absolute coefficient 
           Fresnel_coeff(ienergy)  =  Real(n2*ct/n1/c0)*Abs(Transmi)**2
        end select
        
     End Do
   End Subroutine diff_tran_coef_constitutive
   
   !---------------------------------------------------------------
   Subroutine diff_ref_coef_constitutive_all(Fresnel_coeff,surf_const_coef,param)
     ! Calculates the differential reflectivity from knowledge of the
     ! surface constitutive coefficients up to second order
     ! See Eq. 4.18 Bedeaux's book
     Implicit None
     Type(parameters)            :: param
     Type(surface_constitutive)  :: surf_const_coef(:), chi
     Real(wp)                    :: Fresnel_coeff(:)
     Integer                     :: ienergy
     Real(wp)                    :: c0,s0,ooc
     Complex(wpc)                :: e1,n1,e2,n2,st,ct,Reflec,fresnel
     Complex(wpc)                :: factor(2),tmp(4)

     ! Some energy independent parameters
     e1 = Real(param%eps_vacuum)
     n1 = Sqrt(e1)
     c0 = Cos(param%theta0)
     s0 = Sin(param%theta0)

     Do ienergy = 1,param%Nenergy
        ! Omega over c (dimensionless)
        ooc            =  param%R * param%energy(ienergy) / (param%hbar*param%c)
        chi            =  surf_const_coef(ienergy)
        e2             =  param%eps_substrate(ienergy)
        n2             =  Sqrt(e2) 
        st             =  n1/n2*s0
        ct             =  Sqrt(1._wp - st**2) 
        
        Select case (trim(adjustl(param%polarization)))
        Case('p')
           tmp(1)         =  imu*ooc                                                       
           tmp(2)         =  -tmp(1)**2/4._wp*e1*chi%beta*chi%gamma*s0**2			  
           tmp(3)         =  chi%gamma*c0*ct + n1*n2*chi%beta*e1*s0**2            
           tmp(4)         =  chi%gamma*c0*ct - n1*n2*chi%beta*e1*s0**2    
           factor(1)      =  n2*c0*(1._wp-ooc**2*chi%tau) + n1*ct*(1._wp+ooc**2*chi%tau)
           factor(2)		 =  n2*c0*(1._wp-ooc**2*chi%tau) - n1*ct*(1._wp+ooc**2*chi%tau)
           fresnel        =  (n2*c0-n1*ct)/(n2*c0+n1*ct)  
           Reflec         =  (factor(2) - tmp(1)*tmp(4) -tmp(2)*(n2*c0-n1*ct))/   &
                (factor(1) - tmp(1)*tmp(3) -tmp(2)*(n2*c0+n1*ct))
        Case('s')
           tmp(1)         =  imu*ooc 
           factor(1)      =  n1*c0*(1._wp+ooc**2*chi%tau) + n2*ct*(1._wp-ooc**2*chi%tau)
           factor(2)		  = n1*c0*(1._wp+ooc**2*chi%tau) - n2*ct*(1._wp-ooc**2*chi%tau)
           fresnel        =  (n1*c0-n2*ct)/(n1*c0+n2*ct) 
           Reflec         =  ( factor(2) + tmp(1)*chi%gamma )   /   &
                ( factor(1) - tmp(1)*chi%gamma )  
        End select
        
        Select case(param%out)
        Case('DR')
           ! The differential Reflection 
           Fresnel_coeff(ienergy)  = (Abs(Reflec/fresnel))**2-1._wp
        Case('R')
           ! Absolute coefficient
           Fresnel_coeff(ienergy)  = (Abs(Reflec))**2
        End select
        
     End Do
     Return
     
   End Subroutine diff_ref_coef_constitutive_all
   
   
   !---------------------------------------------------------------
   
   Subroutine diff_tran_coef_constitutive_all(Fresnel_coeff,surf_const_coef,param)
     ! Calculates the differential Transmittance from knowledge of the
     ! surface constitutive coefficients up to second order
     
     Implicit None
     Type(parameters)            :: param
     Type(surface_constitutive)  :: surf_const_coef(:), chi
     Real(wp)                    :: Fresnel_coeff(:)
     Integer                     :: ienergy
     Real(wp)                    :: c0,s0,ooc
     Complex(wpc)                :: e1,n1,e2,n2,st,ct
     Complex(wpc)                :: Transmi,fresnel
     Complex(wpc)                :: factor,tmp(3)

     Write(6,*)
     Write(6,*) '---------------------------------------------------------'
     Write(6,*) ' Warning (subroutine : diff_tran_coef_constitutive_all) !'
     Write(6,*) ' The transmission coefficient is only valid for'
     Write(6,*) ' non-absorbing substrate above the total reflection'
     Write(6,*) '---------------------------------------------------------'
     
     ! Some energy independent parameters
     e1 = Real(param%eps_vacuum)
     n1 = Sqrt(e1)
     c0 = Cos(param%theta0)
     s0 = Sin(param%theta0)
     
     Do ienergy = 1,param%Nenergy
        ! Omega over c (dimensionless)
        ooc            =  param%R * param%energy(ienergy) / (param%hbar*param%c)
        chi            =  surf_const_coef(ienergy)
        e2             =  param%eps_substrate(ienergy)
        n2             =  Sqrt(e2) 
        st             =  n1/n2*s0
        If(Real(st)>1._wp) Then
           Write(6,*) 'Total reflection at : ',param%energy(ienergy),' eV !'
           Write(6,*) '(subroutine : diff_tran_coef_constitutive)'
        Endif
        ct             =  Sqrt(1._wp - st**2) 
        tmp(1)         =  imu*ooc                                              
        tmp(2)         =  -tmp(1)**2/4._wp*e1*chi%beta*chi%gamma*s0**2         
        tmp(3)         =  chi%gamma*c0*ct + n1*n2*chi%beta*e1*s0**2    
        
        select case (trim(adjustl(param%polarization)))
        case('p')
           factor         =  n2*c0*(1._wp-ooc**2*chi%tau)+n1*ct*(1._wp+ooc**2*chi%tau) 
           fresnel        =  2*n1*c0 / (n2*c0 + n1*ct)  
           Transmi        =  2*n1*c0* (1+tmp(2)) /   &
                (factor -tmp(1)*tmp(3) -tmp(2)*(n2*c0 + n1*ct))
        case('s')
           factor         =  n1*c0*(1._wp+ooc**2*chi%tau)+n2*ct*(1._wp-ooc**2*chi%tau) 
           fresnel        =  2*n1*c0 / (n1*c0 + n2*ct) 
           Transmi         =  2*n1*c0   /   &
                ( factor - tmp(1)*chi%gamma )  
        end select
        
        select case(param%out)
        case('DT')
           ! The differential Transmission 
           Fresnel_coeff(ienergy)  = Abs(Transmi/fresnel)**2-1._wp
        case('T')
           ! The absolute coefficient 
           Fresnel_coeff(ienergy)  =  Real(n2*ct/n1/c0)*Abs(Transmi)**2
        end select
        
     End Do
   End Subroutine diff_tran_coef_constitutive_all
   
   !---------------------------------------------------------------
   Subroutine Invariants_calc(Invar,surf_const_coef,param) 
     ! Calculates The (diemsionless) invariants (assuming non-magnetic materials)
     ! See Bedeaux book Eq. (3.9.4)
     Implicit None
     Type(parameters)           :: param
     Type(surface_constitutive) :: surf_const_coef(:), chi
     Type(Invariants)           :: Invar(:)
     Integer                    :: ienergy
     Real(wp)                   :: ooc
     Complex(wpc)               :: e1,e2
     
     ! Some energy independent abbreviations
     e1       = param%eps_vacuum      
     ! Main loop over energy
     Do ienergy = 1,param%Nenergy    
        ooc    =   param%R * param%energy(ienergy) / (param%hbar*param%c)
        chi    =   surf_const_coef(ienergy)
        e2     =   param%eps_substrate(ienergy)
        Invar(ienergy)%e        = chi%gamma  - e2*e1*chi%beta
        Invar(ienergy)%delta_e  = chi%delta -  0.5_wp*(e2+e1)/(e2-e1)*chi%gamma*chi%beta
        Invar(ienergy)%tau      = chi%tau - 0.5_wp*chi%gamma**2/(e2-e1)
        Invar(ienergy)%c        = Aimag(chi%gamma/(e2-e1))  
     End Do
     Return
   End Subroutine Invariants_calc
   
  !---------------------------------------------------------------

  Subroutine diff_ref_coef_invariants(Fresnel_coeff,Invar,param) 
    ! Calculates the differential reflectivity from knowledge of the
    ! invariants.
    ! This method is described in detail in Bedeaux book
    ! See in particular Eq. (4.58) (4.62) 
    Implicit None     
    Type(parameters)  :: param
    Real(wp)          :: Fresnel_coeff(:)
    Type(Invariants)  :: Invar(:), I
    Integer           :: ienergy
    Real(wp)          :: c0,s0,ooc
    Real(wp)          :: fresnel, R
    Complex(wpc)      :: prefactor,nominator,denominator,tmp(2)
    Complex(wpc)      :: ct, st, n2, n1, e2, e1 
    
    ! Some energy independent abbreviations
     e1 = Real(param%eps_vacuum)
     n1 = Sqrt(e1)
     c0 = Cos(param%theta0)
     s0 = Sin(param%theta0)
     
     Do ienergy = 1,param%Nenergy
        ! Some energy dependent abbreviations
        e2        = param%eps_substrate(ienergy)
        n2        = Sqrt(e2) 
        st        = n1/n2*s0                    ! Snell's law
        ct        = Sqrt(1._wp - st**2)
        I         = Invar(ienergy)
        ! Omega over c (dimensionless)
        ooc       = param%R * param%energy(ienergy) / (param%hbar*param%c)

        Select case(trim(adjustl(param%polarization)))
           
        Case('p')
           ! The Fresnel reflectivity
           fresnel =  Abs( (n2*c0-n1*ct)/(n2*c0+n1*ct) )**2  
           !---------------------------------------------------------
           !--- Calculates the Reflection coefficient; Eq. (4.62) 
           !---------------------------------------------------------
           tmp(1) = 2*ooc**2*( Real(I%tau)-Real(I%delta_e)*(n1*s0)**2 )     &
                *( Abs(n2)**2*c0**2-n1**2*Abs(ct)**2 )
           tmp(2) = Aimag(n1*Conjg(n2)*c0*ct)  *  &
                4*ooc**2*( Aimag(I%tau)-Aimag(I%delta_e)*(n1*s0)**2 )
           prefactor   = Exp( 4*ooc*I%c*n1*c0 )
           nominator   = Abs( n2*c0-n1*ct-imu*ooc*(n1/n2)*I%e*s0**2 )**2  &
                - tmp(1) + tmp(2)   
           denominator = Abs( n2*c0+n1*ct+imu*ooc*(n1/n2)*I%e*s0**2 )**2  &
                - tmp(1) - tmp(2)   
           ! The reflection coefficient for p-polarization
           R         =  Real(prefactor * nominator / denominator)
           
        Case('s')
           ! The Fresnel reflectivity
           fresnel =  Abs( (n1*c0-n2*ct)/(n1*c0+n2*ct) )**2  
           !---------------------------------------------------------
           !--- Calculates the Reflection coefficient; Eq. (4.56) 
           !---------------------------------------------------------
           prefactor   = Exp( 4*ooc*I%c*n1*c0 )
           tmp(1) = 2*ooc**2*Real(I%tau)*( (n1*c0)**2-Abs(n2*ct)**2)
           tmp(2) = 4*ooc**2*Aimag(I%tau)*n1*c0*Aimag(n2*ct)
           denominator = Abs(n1*c0+n2*ct)**2 + tmp(1) + tmp(2)
           nominator   = Abs(n1*c0-n2*ct)**2 + tmp(1) - tmp(2)
           ! The reflection coefficient for s-polarization
           R         =  Real(prefactor * nominator / denominator)
           
        End select
        
        Select Case(param%out)
           ! The differential reflection
        Case('DR')
           Fresnel_coeff(ienergy) =   R/fresnel -1._wp 
           ! The reflection coefficient
        Case('R')
           Fresnel_coeff(ienergy) =   R
        End Select
     Enddo
     
     Return
   End Subroutine diff_ref_coef_invariants
   
  !---------------------------------------------------------------
  
   Subroutine diff_tran_coef_invariants(Fresnel_coeff,Invar,param) 

     ! Calculates the differential trasmittance from knowledge of the
     ! invariants.
     ! This method is described in detail in Bedeaux book
     ! See in particular Eq. (4.60) (4.66)
     Implicit None     
     Type(parameters)  :: param
     Real(wp)          :: Fresnel_coeff(:)
     Type(Invariants)  :: Invar(:), I
     Integer           :: ienergy
     Real(wp)          :: c0,s0,ooc
     Real(wp)          :: fresnel, T
     Complex(wpc)      :: prefactor,nominator,denominator,tmp(2)
     Complex(wpc)      :: ct, st, n2, n1, e2, e1 

     Write(6,*)
     Write(6,*) '----------------------------------------------------'
     Write(6,*) ' Warning (subroutine : diff_tran_coef_invariants) !'
     Write(6,*) ' The transmission coefficient is only valid for'
     Write(6,*) ' non-absorbing substrate above the total reflection'
     Write(6,*) '----------------------------------------------------'
     
     ! Some energy independent abbreviations
     e1 = Real(param%eps_vacuum)
     n1 = Sqrt(e1)
     c0 = Cos(param%theta0)
     s0 = Sin(param%theta0)
     
     Do ienergy = 1,param%Nenergy
        ! Some energy dependent abbreviations
        e2        = param%eps_substrate(ienergy)
        n2        = Sqrt(e2) 
        st        = n1/n2*s0                    ! Snell's law
        If(Real(st)>1._wp) Then
           Write(6,*) 'Total reflection at : ',param%energy(ienergy),' eV !'
           Write(6,*) '(subroutine : diff_tran_coef_invariants)'
        Endif
        ct        = Sqrt(1._wp - st**2)
        I         = Invar(ienergy)
        ! Omega over c (dimensionless)
        ooc       = param%R * param%energy(ienergy) / (param%hbar*param%c)
        
        Select case(trim(adjustl(param%polarization)))
           
        Case('p')
           ! The Fresnel transmission coefficient
           fresnel =  Real(4*n1*n2*c0*ct)/( Abs(n2*c0+n1*ct)**2 )  
           !---------------------------------------------------------
           !--- Calculates the transmission coefficient; Eq. (4.66)
           !---------------------------------------------------------
           prefactor   = exp(2*ooc*I%c*(n1*c0-n2*ct))
           nominator   = 4*n1*n2*c0*ct
           tmp(1)		= n2*c0+n1*ct
           tmp(2)		= e1/e2*Abs(I%e)**2*s0**4-2*(Real(I%tau)-Real(I%delta_e)*&
                e1*s0**2)*(e2*c0**2-e1*ct**2)
           denominator = tmp(1)**2-2*ooc*n1/n2*Aimag(I%e)*s0**2*tmp(1)+ooc**2*tmp(2)
           ! The transmission coefficient for p-polarization
           T         =  Real(prefactor * nominator / denominator)
           
        Case('s')
           ! The Fresnel transmission coefficient
           fresnel =  Real(4*n1*n2*c0*ct)/( Abs(n1*c0+n2*ct)**2 )  
           !---------------------------------------------------------
           !--- Calculates the transmission coefficient; Eq. (4.60)
           !---------------------------------------------------------
           tmp(1)		= n1*c0+n2*ct
           tmp(2)		= n1*c0-n2*ct
           prefactor	= exp(2*ooc*I%c*tmp(2))
           nominator	= 4*n1*n2*c0*ct
           denominator = tmp(1)**2-2*ooc**2*Real(I%tau)*(e2-e1)
           ! The transmission coefficient for s-polarization
           T         =  Real(prefactor * nominator / denominator)
           
        End select
        
        Select Case(param%out)
           ! The differential reflection
        Case('DT')
           Fresnel_coeff(ienergy) =   T/fresnel -1._wp 
           ! The reflection coefficient
        Case('T')
           Fresnel_coeff(ienergy) =   T
        End Select
        
     Enddo
     
     Return
   End Subroutine diff_tran_coef_invariants

  !---------------------------------------------------------------
	
   Subroutine Absorp_coef_invariants(Fresnel_coeff,Invar,param)
     
     ! Calculates the absorption from knowledge of the
     ! invariants.
     ! This method is described in detail in Bedeaux book
     ! See in particular Eq. (4.61) (4.67)
     Implicit None     
     Type(parameters)  :: param
     Real(wp)          :: Fresnel_coeff(:)
     Type(Invariants)  :: Invar(:), I
     Integer           :: ienergy
     Real(wp)          :: c0,s0,ooc
     Real(wp)          :: A
     Complex(wpc)      :: prefactor,nominator,denominator,tmp(2)
     Complex(wpc)      :: ct, st, n2, n1, e2, e1 

     Write(6,*)
     Write(6,*) '----------------------------------------------------'
     Write(6,*) ' Warning (subroutine : Absorp_coef_invariants) !'
     Write(6,*) ' The absorption coefficient is only valid for'
     Write(6,*) ' non-absorbing substrate above the total reflection'
     Write(6,*) '----------------------------------------------------'
     
     ! Some energy independent abbreviations
     e1 = Real(param%eps_vacuum)
     n1 = Sqrt(e1)
     c0 = Cos(param%theta0)
     s0 = Sin(param%theta0)
     
     
     Do ienergy = 1,param%Nenergy
        
        ! Some energy dependent abbreviations
        e2        = param%eps_substrate(ienergy)
        n2        = Sqrt(e2) 
        st        = n1/n2*s0                    ! Snell's law
        If(Real(st)>1._wp) Then
           Write(6,*) 'Total reflection at : ',param%energy(ienergy),' eV !'
           Write(6,*) '(subroutine : Absorp_coef_invariants)'
        Endif
        ct        = Sqrt(1._wp - st**2)
        I         = Invar(ienergy)
        ! Omega over c (dimensionless)
        ooc       = param%R * param%energy(ienergy) / (param%hbar*param%c)
        
        Select case(trim(adjustl(param%polarization)))
           
        Case('p')
           !---------------------------------------------------------
           !--- Calculates the Absorption coefficient; Eq. (4.67) 
           !---------------------------------------------------------
           prefactor   = exp(2*ooc*I%c*(n1*c0-n2*ct))
           nominator   = 4*ooc*n1*c0* &
		( I%c*(e2-e1)*(s0**2+ct**2)-Aimag(I%e)*s0**2+2*ooc/n2*s0**2*ct**2* &
		(I%c**2*(e2-e1)**2-I%c*Aimag(I%e)*(e2-e1)) )
           tmp(1)		= n2*c0+n1*ct
           tmp(2)		= e1/e2*Abs(I%e)**2*s0**4-2*(Real(I%tau)-Real(I%delta_e)*&
                e1*s0**2)*(e2*c0**2-e1*ct**2)
           denominator = tmp(1)**2-2*ooc*n1/n2*Aimag(I%e)*s0**2*tmp(1)+ooc**2*tmp(2)
           A         =  prefactor * nominator / denominator
           
        Case('s')
           !---------------------------------------------------------
           !--- Calculates the Absorption coefficient; Eq. (4.61) 
           !---------------------------------------------------------
           tmp(1)		= n1*c0-n2*ct
           tmp(2)		= ooc*Real(I%tau)*(e2-e1)
           prefactor	= exp(2*ooc*I%c*tmp(1))
           nominator	= 4*n1*c0*tmp(2)
           denominator = tmp(1)**2-2*ooc*tmp(2)
           A			=  prefactor * nominator / denominator
           
        End select
        
        Fresnel_coeff(ienergy) = A
        
     Enddo
     Return
	
   End Subroutine Absorp_coef_invariants
   
   !---------------------------------------------------------------
   Subroutine diff_ref_coef_aspnes(Fresnel_coeff,surf_const_coef,param) 
     ! Calculates the differential reflectivity from knowledge of the
     ! surafce constitutive coefficients.
     ! This method is described in the article of Yves Borensztein
     ! Thin Solid Films 125 (1985) 129   6-7
     Implicit None     
     Type(parameters)              :: param
     Real(wp)                      :: Fresnel_coeff(:)
     Type(surface_constitutive)    :: surf_const_coef(:),chi
     Integer                       :: ienergy
     Real(wp)                      :: c0,s0,ooc
     Complex(wpc)                  :: e1,e2,n1,y1,y2,prefactor
     
     
     ! Some energy independent parameters
     e1 = param%eps_vacuum
     n1 = Sqrt(e1)
     c0 = Cos(param%theta0)
     s0 = Sin(param%theta0)
 
     Do ienergy = 1,param%Nenergy
        ! Some energy dependent abbreviations
        e2        =  param%eps_substrate(ienergy)
        ! 1/lambda dimensionless
        ooc		  =   param%R * param%energy(ienergy) / &
             (2*pi * param%hbar * param%c)
        chi		  =  surf_const_coef(ienergy)
        prefactor =  8*pi*c0*n1*ooc
        ! Differential reflectivity
        Select case (trim(adjustl(param%polarization)))
        case('s')
           Fresnel_coeff(ienergy) =  aimag(prefactor*chi%gamma/(e2-e1))
        case('p')
           y1 = (e2-e1*s0**2)*chi%gamma - e2**2*e1*s0**2*chi%beta
           y2 = (e1-e2)*(e1*s0**2 - e2*c0**2)
           Fresnel_coeff(ienergy) =  aimag(prefactor*y1/y2) 
        End select
     Enddo
     
   End Subroutine diff_ref_coef_aspnes

   !------------------------------------------------------------------------

End Module Optics_mod
