
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: interaction_mod.f90,v 1.1 2001/07/29 12:40:20 ingves Exp ingves $
!---------------------------------------------------------------------------------




Module Interaction_mod


  Use Global_definitions
  Use Legendre_mod, Only : gausslegendre


Contains

  !---------------------------------------------------------------

   Function lattice_sum(d_mu,n,param)	Result(S)
     !-----------------------------------------------
     ! See Physica A143, 164 (1987); Eq. (3.4)
     ! Bedeaux book P263 Eqs. (10.9) (10.10)
     !-----------------------------------------------
     
     !-----------------------------------------------
     ! d_mu=distance between a multipole and the surface
     !-----------------------------------------------
     Implicit None
     Type(parameters)   :: param
     Integer            :: n
     Real(wp)           :: d_mu,S
     Real(wp)           :: d,norm,Lmax
     
     !-----------------------------------------------
     ! d=distance between a multipole and its image
     ! normalized (dimensionless) by L
     !----------------------------------------------- 
     
     ! Nature of the network
     Select Case(param%network)
        
     Case('square')
	d = 2*d_mu*param%R/param%lattice_const
	S = Sr_square(d,n,param%Levels)
        
     Case('hexagonal')
	d = 2*d_mu*param%R/param%lattice_const
	S = Sr_hexagonal(d,n,param%Levels)	
        
     Case('MFT','RPT')
	! Normalization factor
	norm  =	1._wp/sqrt(param%density)/(2*param%Rapparent)
	! Dimensionless by normalization with 2*Rapp
	d     =	2*d_mu*param%R/(2*param%Rapparent)
	! Dimensionless value for the maximum distance between clusters
	Lmax  =	param%Levels*norm
	S     = 2*pi*norm**(n-1)*Ir_random(d,n,Lmax,param%Nint,param%Rapparent)
        
     Case Default
	Write(6,*) 'Error in the nature of network : ',param%network, &
             ' not supported yet !'
	Pause
	stop
     End Select
     
     Return
     
   End Function lattice_sum

!---------------------------------------------------------------

   Subroutine Renorm_Polarizability(d_mu,param,alpha)
     !------------------------------------------------------
     ! Renormalization of the polarizabibities by the scheme
     ! of Barrera PRB 43-17 P13819 (1991)
     !------------------------------------------------------
     
     Implicit None
     Type(parameters)      ::  param
     Complex(wpc)          ::  alpha(2,param%Nenergy)
     Real(wp)              ::  d_mu
     Complex(wpc)          ::  pol_old(2),pol_new(2)
     Complex(wp)           ::  Fxx,Fzz,Fxz
     Real(wp)              ::  Integral(6)
     Real(wp)              ::  d,phi,error(2),Lmax,norm
     Complex(wpc)          ::  e1,e2,eps
     Integer               ::  N,ienergy
     Real(wp),Parameter    ::  epsilon=1e-6
     Integer,Parameter     ::  nitmax=35
     Integer               ::  nit
     
     ! Some energy independent abbreviations
     e1    =  param%eps_vacuum
     N     =  param%Nenergy
     phi   =  param%coverage
     norm  =  param%R/param%Rapparent
     
     ! Dimensionless polarizability
     alpha(:,:)	=  alpha(:,:)*norm
     
     ! Dimensionless distance between multipoles and surface
     d =  d_mu*norm
     
     ! Dimensionless value for the maximum distance between clusters
     Lmax  =  param%Levels/sqrt(param%density)/(2*param%Rapparent)
     
     ! Evaluation of the integrals Fxx,Fzz,Fxz
     Call Integral_RPT(d,Lmax,param%Rapparent,Integral,param%Nint)
     
     
     ! Self consistent evaluation of the renormalized polarizabilities
     Do ienergy=1,N
        
        e2  =  param%eps_substrate(ienergy)
        If(param%above) then
           alpha(:,ienergy)  =  alpha(:,ienergy)/(4*pi*e1)
           eps               =  (e2-e1)/(e2+e1)
        Else
           alpha(:,ienergy)  =  alpha(:,ienergy)/(4*pi*e2)
           eps               =  (e1-e2)/(e2+e1)
        Endif
        
        Fxx  = 5*phi*( Integral(1)-2*eps*Integral(2)+eps**2*Integral(3) )/4
        Fzz  = phi*( Integral(1)+2*eps*Integral(4)+eps**2*Integral(5) )/2
        Fxz  = -9*phi*( eps**2*Integral(6) )/16
        
        !index	 1 : parallel component
	!	 2 : perpendicular component
        pol_old(:)  =   alpha(:,ienergy)
        error(:)    =   1._wp
        nit         =   1
        Do While (error(1)>=epsilon.or.error(2)>=epsilon)
           pol_new(1) = 1._wp+Fxx/4*pol_old(1)**2+Fxz*pol_old(1)*pol_old(2)
           pol_new(2) = 1._wp+2*Fxz*pol_old(1)*pol_old(2)+Fzz*pol_old(2)**2/4
           pol_new(:) = pol_new(:)*alpha(:,ienergy)
           error(:)   = abs((pol_new(:) - pol_old(:))/pol_old(:))
           pol_old(:) = pol_new(:)
           nit        = nit+1
           If(nit>=nitmax) Goto 10 
        Enddo
        !	 Write(61,'(10f20.12)') pol_old(1),alpha(1,ienergy),error(:)
        
        ! Renormalized polarizabilities
10      If(param%above) then
           alpha(:,ienergy) = 4*pi*e1*pol_old(:)/norm
        Else
           alpha(:,ienergy) = 4*pi*e2*pol_old(:)/norm
        Endif
        
     Enddo
     
     Return
     
   End Subroutine Renorm_Polarizability

!---------------------------------------------------------------

   Subroutine Integral_RPT(d,Lmax,Rapp,Integral,Nint)
     
     !-----------------------------------------------------
     ! Routine for evaluation of the integrals Fxx Fxz Fzz
     ! in the Barrera's renormalization scheme
     ! PRB 43-17 (1991)
     !-----------------------------------------------------
     
     Implicit None
     Real(wp)							:: Lmax,d,Integral(6),Rapp
     Integer								:: Nint
     Real(wp), Allocatable				:: w(:),x(:),y(:),r(:),integrand(:)	
     
     Allocate(x(Nint),y(Nint),r(Nint),w(Nint),integrand(Nint))
     
     ! Calculate the points and the weights for GaussLegendre integration
     Call gausslegendre(1._wp,Lmax,x,w)
     ! Value of pair correlation function
     y = Pair_Correlation(2._wp*Rapp*x)
     ! Value for the distance function
     r = sqrt(x**2+d**2)
     
     ! The integrals		
     ! Integral(1)
     integrand = y/(x**5)
     Integral(1) = Sum(w*integrand)
     ! Integral(2)
     integrand = y*(x**2-d**2/5)/(x**2)/(r**5)
     Integral(2) = Sum(w*integrand)
     ! Integral(3)	
     integrand = y*x*( x**4-2*(d*x)**2/5+2*d**4/5 )/(r**10)
     Integral(3) = Sum(w*integrand)
     ! Integral(4)
     integrand = y*(x**2-2*d**2)/(x**2)/(r**5)
     Integral(4) = Sum(w*integrand)
     ! Integral(5)
     integrand = y*x*(x**4-4*(d*x)**2+4*d**4)/(r**10)
     Integral(5) = Sum(w*integrand)
     ! Integral(6)
     integrand =  y*d**2*x**3/(r**10)
     Integral(6) = Sum(w*integrand)
     
     Deallocate(x,y,r,w,integrand)
     
   End Subroutine Integral_RPT
   
   !---------------------------------------------------------------
   
   Function Sr_square(d,l,level)	Result(Sr)
     
     Implicit None
     Real(wp)							:: Sr,d
     Integer								:: level,l
     Real(wp)							:: S,SD,SM,r 
     Integer								:: m,n
     
     ! Initiate
     SM = 0._wp
     SD = 0._wp
     S  = 0._wp
     Do m = level,1,-1
        ! Ths sum along the 1st axis
        r = sqrt(m**2 + d**2)
        SM  = SM + Argument(l,d,r)
        ! Sum along the diagonal
        r = sqrt(2*(m**2) + d**2)
        SD  = SD + Argument(l,d,r)
        ! Sum over points in the lower triangle of the 1st qadrant
        S = 0._wp
        Do n = m,0,-1
           r = sqrt(m**2 + n**2 + d**2)
           S = S + Argument(l,d,r)
        Enddo
     Enddo
     ! Sum in the first quadrant 
     Sr = 2*S - SD - SM
     ! The total lattice sum
     Sr = 4*Sr
     ! Prefactor of spherical harmonics
     Sr = Sr*sqrt((2*l+1._wp)/(4*pi))
     
   End Function Sr_square
   
 !---------------------------------------------------------------
 	
   Function Sr_hexagonal(d,l,level)	Result(Sr)
     
     Implicit None
     Real(wp)    ::  d,Sr
     Integer     :: level,l
     Real(wp)    :: SD,S,r
     Integer     :: m,n
     
     ! Initiate
     SD = 0._wp
     S  = 0._wp
     
     Do m = level,1,-1
        ! Sum along the diagonal
        r = sqrt(3*(m**2) + d**2)
        SD  = SD + Argument(l,d,r)
     Enddo
     ! Sum over points in the 1st quadrant
     S = 0._wp
     Do m = level,1,-1
        Do n= -m+1,m-1,1
           r = sqrt(m**2 + n**2 + m*n + d**2)
           S = S + Argument(l,d,r)
        Enddo
     Enddo
     ! The total lattice sum
     Sr = 4 * (SD + S)
     ! Prefactor of spherical harmonics
     Sr = Sr*sqrt((2*l+1._wp)/(4*pi))
     
   End Function Sr_hexagonal
   
   !-----------------------------------------------------------------
   
   Function Ir_random(d,l,Lmax,Nint,Rapp)		Result(Ir)
     
     Implicit None
     Real(wp)                 :: Ir,d,Lmax,Rapp
     Integer                  :: Nint,l					
     Real(wp), Allocatable    :: weight(:),r(:),integrand(:)
     Integer                  :: i
     
     
     ! Formulas 10-13 10-14 + P269
     ! The integral is dimensionless (unit 'L/2*Rapp')
     ! Evaluation of the integrals Ir by Gauss Legendre integration
     
     Allocate(r(Nint),integrand(Nint),weight(Nint))
     Call gausslegendre(1._wp,Lmax,r,weight)
     integrand	=  Pair_Correlation(2._wp*Rapp*r)
     Do i=1,Nint
        integrand(i)=integrand(i)*r(i)*Argument(l,d,sqrt(r(i)**2+d**2))
     Enddo
     Ir	=  Sum(weight * integrand)
     Ir =  Ir*Sqrt( (2*l+1._wp)/(4*pi) )
     Deallocate(r,integrand,weight)
     
   End Function Ir_random
   
   !---------------------------------------------------------------
   Function Argument(l,d,r)	Result(Arg)
     
     Implicit None
     Real(wp)   ::  d,r,Arg
     Integer    ::  l
     Real(wp)   ::  cosi
     
     cosi = d/r
     Select Case(l)
     Case(2)
	! P(2,0)/r**3
	Arg = ( 3*cosi**2 - 1._wp)/2/(r**3)
     Case(3)
	! P(3,0)/r**3
	Arg = ( 5*cosi**3 - 3*cosi)/2/(r**4)
     Case(4)
	! P(4,0)/r**3
	Arg = ( 35*cosi**4 -30*cosi**2 + 3._wp)/8/(r**5) 
     End Select
     Return
     
   End Function Argument
   
   !---------------------------------------------------------------

   Function Pair_Correlation(x)  Result(g)
     ! Unit : nanometer like Rapp (dimension)
     
     Implicit None
     Real(wp)   ::  x(:),g(Size(x,1))
     
     g = 1._wp
     
   End Function Pair_Correlation
   
   
 End Module Interaction_mod
 
 
 
