
!--------------------------------------------------------------------------------
!              SPHEROID part of GranFilm v 1.0
!
!   (C) Ingve Simonsen and Remi Lazzari, 1998-2001	
!
!   $Id: legendre.f90,v 1.1 2001/07/29 12:40:56 ingves Exp ingves $
!---------------------------------------------------------------------------------




Module Legendre_mod

  Use Global_definitions

  !---------------------------------------------------------------------
  ! The sign convension of Morse and Feshbach is used for the functions
  !
  ! The stable recurrence relations for evaluating the polynomials are 
  ! from :  OLVER AND SMITH,J.COMPUT.PHYSICS,51(1983),N0.3,502-518
  !---------------------------------------------------------------------
  
Contains
  
  
  ! ----------------------------------------------------------
  !  Legendre Functions of First Kind For Real Argument
  ! ----------------------------------------------------------
  
  Subroutine LegendreP(P,dPdx,x,lmax)
    ! Calculates the Associated Legendre function of the 1st kind
    ! for vector input
    ! INPUT  :    x         : the real argument between (-1,1)
    !             lmax      : the maximum degree
    ! OUTPUT :    P         : the associated legendre function of the 1st kind
    !                         for degrees 0...lmax and order m=0,1
    !             dPdx      : the derivative of the associated legendre function
    ! 
    
    
    Implicit none
    ! INPUT
    Integer          ::	 lmax
    Real(wp)         ::	 x(:)
    Real(wp)         ::	 P(size(x),0:lmax,0:1)
    Real(wp)         ::	 dPdx(size(x),0:lmax,0:1)  
    ! Local
    Integer          ::  l,ix,m
    Real(wp)         ::  y,z
    
    
    !-----------------------
    ! Start loop over x
    !-----------------------
    Do ix = 1,size(x)
       y=x(ix)
       z=1-y**2
       
       !-----------------------
       ! Start loop over m
       !-----------------------
       Do m=0,1
          
          ! Initial values for degree l=m and m+1
          ! Notice that we only need these functions for m=0,1
          Select case(m)
          Case(0) 
             P(ix,0,0)    = 1._wp   
             P(ix,1,0)    = y
             dPdx(ix,0,0) = 0._wp
          Case(1)
             P(ix,0,1)	= 0._wp      ! Strictly speaking P(l,m) is not defined for l<m, 
             dPdx(ix,0,1) = 0._wp		 ! but we define for convenience p(l,m)=0 
             ! in this case. 
             P(ix,1,1)	= sqrt(z)          
             P(ix,2,1)	= 3*y*sqrt(z)          
             dPdx(ix,1,1) = -y/sqrt(z)          
          Case default
             ! Error 
             write(*,*) 'ERROR : Illegal value for m in LegendreP !'
             pause
             stop
          End select
          
          ! Perfoms forward l-wise recurrence to obtain
          ! P(m,0,x),...,P(m,lmax,x)
          Do l=1+m,lmax-1
             P(ix,l+1,m)  = ( (2*l+1)*y*P(ix,l,m) - (l+m)*P(ix,l-1,m) )/(l-m+1)  
             dPdx(ix,l,m) = ( -l*y*P(ix,l,m)+(l+m)*P(ix,l-1,m) )/z  
          Enddo
          ! Calculates the top level for dPdx
          l            =  lmax 
          dPdx(ix,l,m) = ( -l*y*P(ix,l,m)+(l+m)*P(ix,l-1,m) )/z
          
       Enddo
       !-----------------------
       ! End loop over m
       !-----------------------
       
    Enddo
    !-----------------------
    ! End loop over x
    !-----------------------

  End Subroutine LegendreP


  !---------------------------------------------------------------------


  Function LegendreP_R(x,l,m)		Result(P)
    ! Calculates the Associated Legendre function of the 1st kind
    ! for real input
    ! INPUT  :    x         : the real argument between (-1,1)
    !             l,m		: the degree
    ! OUTPUT :    P         : the associated legendre function of the 1st kind
    
    Implicit none
    ! INPUT
    Real(wp),Intent(in)  ::  x
    Integer,Intent(in)   ::  l,m
    ! Output
    Real(wp)             ::  P
    ! Local
    Integer              ::  i
    Real(wp)             ::  P1,P2,P3
	            
    ! Initial values for degree l=m and m+1
    ! Notice that we only need these functions for m=0,1
    Select case(m)
     Case(0) 
        P1   =   1._wp  
        P2   =   x
      Case(1) 
        P1   =  sqrt(1-x**2)        
        P2   =  3*x*sqrt(1-x**2)               
      Case default
         write(*,*) 'ERROR : Illegal value for m in LegendreP_R !'
         pause
         stop
      End select
      ! Get out for the first values
      If(l-1<1+m) Then
	 Select Case(m)
	 Case(0)
            If(l==0) P = P1
            If(l==1) P = P2
	 Case(1)
            If(l==0) P = 0._wp
            If(l==1) P = P1
            If(l==2) P = P2
	 End Select
	 Return
      Endif
      ! Perfoms forward l-wise recurrence to obtain P(l,m)(x)
      Do i=1+m,l-1
         P3  = ( (2*i+1)*x*P2 - (i+m)*P1 )/(i-m+1)
         P1  =	P2
         P2  = P3  
      Enddo
      P = P3
      Return
      
    End Function LegendreP_R

 !---------------------------------------------------------------------

    Function LegendredP_R(x,l,m)   Result(dPdx)
      ! Calculates the derivate of the associated Legendre function of the 1st kind
      ! for real input
      ! INPUT  :    x         : the real argument between (-1,1)
      !             l,m	      : the degree
      ! OUTPUT :    dPdx      : the result
      
      Implicit none
      ! INPUT
      Real(wp),Intent(in)   ::  x
      Integer,Intent(in)    ::  l,m
      ! Output
      Real(wp)              ::  dPdx
      
      ! Get out for the first terms	
      If(l-1<m) Then
	 SelectCase(m)
	 Case(0)
            If(l==0) dPdx = 0._wp
	 Case(1)
            If(l==0) dPdx = 0._wp
            If(l==1) dPdx = -x/sqrt(1-x**2)
	 End Select
	 Return
      Endif
      ! The formula
      dPdx = ( l*x*LegendreP_R(x,l,m)-(l+m)*LegendreP_R(x,l-1,m) )/(x**2-1)
      Return	            
      
    End Function LegendredP_R


  ! ----------------------------------------------------------
  ! --- Routines for Gauss-Legendre integration
  ! ----------------------------------------------------------

    Subroutine GaussLegendre(x1,x2,x,w)

      Implicit None
      Real(wp)       :: x1,x2
      Real(wp)       :: x(:),w(:)
      
      ! Calculates the weights and abcissa for Gauss-Legendre integration 
      Select Case(wp)
      Case(sp)
         !------------------------   
         ! ---Single precision
         !------------------------   
         Call gauleg_sp(x1,x2,x,w)
      Case(dp)
         !------------------------   
         ! ---Double precision
         !------------------------   
         Call gauleg_sp(x1,x2,x,w)
      End Select
      
      Return
    End Subroutine GaussLegendre
    
    !-----------------------------------------------------------------------

    ! Single precision version from Numerical Recipes
    Subroutine  Gauleg_sp(x1,x2,x,w)
      
      Implicit None
      Real(wp)               ::	x1,x2,x(:),w(:)
      Real(dp),parameter     ::	EPS=3.d-14
      Integer                ::	i,j,m,n
      ! Use double precision for intermediate steps
      Real(dp)               ::	p1,p2,p3,pp,xl,xm,z,z1
      
      n = size(x)
      m=(n+1)/2
      xm=0.5d0*(x2+x1)
      xl=0.5d0*(x2-x1)
      Do i=1,m
         z=cos(pi*(i-.25d0)/(n+.5d0))
1        continue
         p1=1.d0
         p2=0.d0
         Do j=1,n
            p3=p2
            p2=p1
            p1=((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
         Enddo
         pp=n*(z*p1-p2)/(z*z-1.d0)
         z1=z
         z=z1-p1/pp
         If(abs(z-z1).gt.EPS) goto 1
         x(i)=xm-xl*z
         x(n+1-i)=xm+xl*z
         w(i)=2.d0*xl/((1.d0-z*z)*pp*pp)
         w(n+1-i)=w(i)
      Enddo
      Return
      
    End Subroutine Gauleg_sp
    
    !-----------------------------------------------------------------------

    ! Double precision version modified
    Subroutine  Gauleg_dp(x1,x2,x,w)
      
      Use FMZM
      Implicit None
      Real(wp)      ::  x1,x2,x(:),w(:)
      Type(FM)      ::	EPS
      Integer       ::  i,j,m,n
      ! Use multiple precision for intermediate steps
      Type(FM)      ::	p1,p2,p3,pp,xl,xm,z,z1
      Type(FM)      ::	val1,val2
      
      n	  = size(x)
      ! Values to keep the precision
      EPS     = TO_FM('3e-20')
      val1    = TO_FM(1)/TO_FM(2)
      val2    = TO_FM(1)/TO_FM(4)
      m=(n+1)/2
      xm=(x2+x1)/2
      xl=(x2-x1)/2
      Do i=1,m
         z=cos(pi*(i-val2)/(n+val1))
1        continue
         p1=1
         p2=0
         Do j=1,n
            p3=p2
            p2=p1
            p1=((2*j-1)*z*p2-(j-1)*p3)/j
         Enddo
         pp=n*(z*p1-p2)/(z*z-1)
         z1=z
         z=z1-p1/pp
         If(abs(z-z1).gt.EPS) goto 1
         x(i)        =	TO_DP(xm-xl*z)
         x(n+1-i)    =	TO_DP(xm+xl*z)
         w(i)        =	TO_DP(2*xl/((1-z*z)*pp*pp))
         w(n+1-i)    =	w(i)
      Enddo
      Return
      
    End Subroutine Gauleg_dp
    

    

    ! ----------------------------------------------------------
    ! --- Routines for Auto-Adaptative Integration Method
    ! --- Quadpack
    ! ----------------------------------------------------------
    
    Subroutine Quadpack(F,tr,Int,param)
      
      ! Subroutine for calling the package of quadrature adaptative 
      ! integration over finite interval
      Implicit None
      Real(wp),External        ::  F
      Real(wp)                 ::  tr
      Type(parameters)         ::  param
      Real(wp)                 ::  Int
      Real(wp)                 ::  epsrel,epsabs,abserr
      Integer,Parameter	       ::  key=4
      Integer                  ::  neval,ier,limit,lenw,last
      Integer,Allocatable      ::  iwork(:)
      Real(wp),Allocatable     ::  work(:)
      
      ! Some initial abreviations
      epsabs   =  param%epsabs
      epsrel   =  param%epsrel
      limit    =  param%limit
      lenw     =  4*limit
      ! Allocation of the working arrays
      Allocate(iwork(limit),work(lenw))
      
      !-----------------------------
      ! Call of the quapack routines
      !-----------------------------
      Select Case(wp)
         !---------------------------
         ! Single precision version
         !---------------------------
      Case(sp)
         Call qag(F,-1._wp,tr,epsabs,epsrel,key,Int,abserr, &
              neval,ier,limit,lenw,last,iwork,work)
         !---------------------------
         ! Double precision version
         !---------------------------
      Case(dp)
         Call dqag(F,-1._wp,tr,epsabs,epsrel,key,Int,abserr, &
              neval,ier,limit,lenw,last,iwork,work)
      End Select
      
      ! Error code in Quadpack
      If(ier/=0) Then
         ! Output of some control parameters
	 Open(53,file='error_int.dat',status='unknown')
	 Write(53,'(i2,3(E15.5),2i5)') &
              ier,Int,abserr/Abs(Int),abserr,neval,last
      Endif
      
      
      ! Deallocation of arrays
      Deallocate(iwork,work)
      
    End Subroutine Quadpack
    
    !-----------------------------------------------------------------------------------------

    Subroutine Quadpack_Infinite(F,bound,Int,param)
     
     ! Subroutine for calling the package of quadrature adaptative 
     ! integration over infinite interval
     Implicit None
     Real(wp),External      ::  F
     Real(wp)               ::  bound
     Type(parameters)       ::  param
     Real(wp)               ::  Int
     Real(wp)               ::  epsrel,epsabs,abserr
     Integer,Parameter      ::  key=4
     Integer                ::  neval,ier,limit,lenw,last
     Integer,Allocatable    ::  iwork(:)
     Real(wp),Allocatable   ::  work(:)
     
     ! Some initial abreviations
     epsabs    =  param%epsabs
     epsrel    =  param%epsrel
     limit     =  param%limit
     lenw      =  4*limit
     ! Allocation of the working arrays
     Allocate(iwork(limit),work(lenw))
     
     !-----------------------------
     ! Call of the quapack routines
     !-----------------------------
     Select Case(wp)
	!---------------------------
	! Single precision version
	!---------------------------
     Case(sp)
	Call qagi(F,bound,1,epsabs,epsrel,Int,abserr,neval, &
             ier,limit,lenw,last,iwork,work)
	!---------------------------
	! Double precision version
	!---------------------------
     Case(dp)
	Call dqagi(F,bound,1,epsabs,epsrel,Int,abserr,neval, &
             ier,limit,lenw,last,iwork,work)
     End Select
     
     ! Error code in Quadpack
     If(ier/=0) Then
	! Output of some control parameters
        Open(53,file='error_int.dat',status='unknown')
        Write(53,'(i2,3(E15.5),2i5)') &
             ier,Int,abserr/Abs(Int),abserr,neval,last
     Endif
     
     ! Deallocation of arrays
     Deallocate(iwork,work)
     
   End Subroutine Quadpack_Infinite
   
 End Module Legendre_mod








